from .. import Provider as BankProvider


class Provider(BankProvider):
    """Bank provider for mk_MK locale (Macedonian).

    Sources:
    - https://en.wikipedia.org/wiki/IBAN#IBAN_formats_by_country
      MK IBAN: MK + 2 check digits + 3-digit bank code + 10-digit account + 2 national check digits
      Total BBAN length: 15 digits/chars (3 bank + 10 account + 2 check)
    - https://en.wikipedia.org/wiki/List_of_banks_in_North_Macedonia
    """

    bban_format = "###????????????##"
    country_code = "MK"

    banks = (
        "Народна банка на Република Северна Македонија",
        "Силк Роуд банка",
        "Халк банка",
        "Шпаркасе банка",
        "Капитал банка",
        "Комерцијална банка",
        "Развојна банка на Северна Македонија",
        "НЛБ банка",
        "ПроКредит банка",
        "Централна кооперативна банка",
        "Алта банка",
        "Стопанска банка",
        "Универзална инвестициона банка",
        "ТТК банка",
    )



import argparse
import itertools
import logging
import os
import random
import sys
import textwrap

from io import TextIOWrapper
from pathlib import Path
from typing import Dict, List, Optional, TextIO, TypeVar, Union

from . import VERSION, Faker, documentor, exceptions
from .config import AVAILABLE_LOCALES, DEFAULT_LOCALE, META_PROVIDERS_MODULES
from .documentor import Documentor
from .providers import BaseProvider

__author__ = "joke2k"

T = TypeVar("T")


def _encode_for_output(value: str, output: TextIO) -> str:
    encoding = getattr(output, "encoding", None)
    if encoding is None:
        return value

    try:
        value.encode(encoding)
    except UnicodeEncodeError:
        return value.encode(encoding, errors="backslashreplace").decode(encoding)

    return value


def print_provider(
    doc: Documentor,
    provider: BaseProvider,
    formatters: Dict[str, T],
    excludes: Optional[List[str]] = None,
    output: Optional[TextIO] = None,
) -> None:
    if output is None:
        output = sys.stdout
    if excludes is None:
        excludes = []

    print(file=output)
    print(_encode_for_output(f"### {doc.get_provider_name(provider)}", output), file=output)
    print(file=output)

    margin = max(30, doc.max_name_len + 2)
    for signature, example in formatters.items():
        if signature in excludes:
            continue
        signature_lines = textwrap.wrap(signature, width=margin, subsequent_indent="  ")
        try:
            lines = textwrap.wrap(
                str(example).expandtabs(),
                width=150 - margin,
                initial_indent="# ",
                subsequent_indent="  ",
            )
        except UnicodeDecodeError:
            # The example is actually made of bytes.
            # We could coerce to bytes, but that would fail anyway when we will
            # try to `print` the line.
            lines = ["<bytes>"]
        except UnicodeEncodeError:
            raise Exception(f"error on {signature!r} with value {example!r}")
        for left, right in itertools.zip_longest(signature_lines, lines, fillvalue=""):
            line = f"\t{left:<{margin}}  {right}"
            print(_encode_for_output(line, output), file=output)


def print_doc(
    provider_or_field: Optional[str] = None,
    args: Optional[List[T]] = None,
    lang: str = DEFAULT_LOCALE,
    output: Optional[Union[TextIO, TextIOWrapper]] = None,
    seed: Optional[float] = None,
    includes: Optional[List[str]] = None,
) -> None:
    if args is None:
        args = []
    if output is None:
        output = sys.stdout
    fake = Faker(locale=lang, includes=includes)
    fake.seed_instance(seed)

    from faker.providers import BaseProvider

    base_provider_formatters = list(dir(BaseProvider))

    if provider_or_field:
        if "." in provider_or_field:
            parts = provider_or_field.split(".")
            locale = parts[-2] if parts[-2] in AVAILABLE_LOCALES else lang
            fake = Faker(locale, providers=[provider_or_field], includes=includes)
            fake.seed_instance(seed)
            doc = documentor.Documentor(fake)
            doc.already_generated = base_provider_formatters
            print_provider(
                doc,
                fake.get_providers()[0],
                doc.get_provider_formatters(fake.get_providers()[0]),
                output=output,
            )
        else:
            try:
                print(fake.format(provider_or_field, *args), end="", file=output)
            except AttributeError:
                raise ValueError(f'No faker found for "{provider_or_field}({args})"')

    else:
        doc = documentor.Documentor(fake)
        unsupported: List[str] = []

        while True:
            try:
                formatters = doc.get_formatters(with_args=True, with_defaults=True, excludes=unsupported)
            except exceptions.UnsupportedFeature as e:
                unsupported.append(e.name)
            else:
                break

        for provider, fakers in formatters:
            print_provider(doc, provider, fakers, output=output)


class Command:
    def __init__(self, argv: Optional[str] = None) -> None:
        self.argv = argv or sys.argv[:]
        self.prog_name = Path(self.argv[0]).name

    def execute(self) -> None:
        """
        Given the command-line arguments, this creates a parser appropriate
        to that command, and runs it.
        """

        # retrieve default language from system environment
        default_locale = os.environ.get("LANG", "en_US").split(".")[0]
        if default_locale not in AVAILABLE_LOCALES:
            default_locale = DEFAULT_LOCALE

        epilog = f"""supported locales:

  {', '.join(sorted(AVAILABLE_LOCALES))}

  Faker can take a locale as an optional argument, to return localized data. If
  no locale argument is specified, the factory falls back to the user's OS
  locale as long as it is supported by at least one of the providers.
     - for this user, the default locale is {default_locale}.

  If the optional argument locale and/or user's default locale is not available
  for the specified provider, the factory falls back to faker's default locale,
  which is {DEFAULT_LOCALE}.

examples:

  $ faker address
  968 Bahringer Garden Apt. 722
  Kristinaland, NJ 09890

  $ faker -l de_DE address
  Samira-Niemeier-Allee 56
  94812 Biedenkopf

  $ faker profile ssn,birthdate
  {{'ssn': u'628-10-1085', 'birthdate': '2008-03-29'}}

  $ faker -r=3 -s=";" name
  Willam Kertzmann;
  Josiah Maggio;
  Gayla Schmitt;

"""

        formatter_class = argparse.RawDescriptionHelpFormatter
        parser = argparse.ArgumentParser(
            prog=self.prog_name,
            description=f"{self.prog_name} version {VERSION}",
            epilog=epilog,
            formatter_class=formatter_class,
        )

        parser.add_argument("--version", action="version", version=f"%(prog)s {VERSION}")

        parser.add_argument(
            "-v",
            "--verbose",
            action="store_true",
            help="show INFO logging events instead "
            "of CRITICAL, which is the default. These logging "
            "events provide insight into localization of "
            "specific providers.",
        )

        parser.add_argument(
            "-o",
            metavar="output",
            type=argparse.FileType("w"),
            default=sys.stdout,
            help="redirect output to a file",
        )

        parser.add_argument(
            "-l",
            "--lang",
            choices=AVAILABLE_LOCALES,
            default=default_locale,
            metavar="LOCALE",
            help="specify the language for a localized provider (e.g. de_DE)",
        )
        parser.add_argument(
            "-r",
            "--repeat",
            default=1,
            type=int,
            help="generate the specified number of outputs",
        )
        parser.add_argument(
            "-s",
            "--sep",
            default="\n",
            help="use the specified separator after each output",
        )

        parser.add_argument(
            "--seed",
            metavar="SEED",
            type=int,
            help="specify a seed for the random generator so "
            "that results are repeatable. Also compatible "
            "with 'repeat' option",
        )

        parser.add_argument(
            "-i",
            "--include",
            action="append",
            help="list of additional custom providers to "
            "user, given as the import path of the module "
            "containing your Provider class (not the provider "
            "class itself)",
        )

        parser.add_argument(
            "fake",
            action="store",
            nargs="?",
            help="name of the fake to generate output for (e.g. profile)",
        )

        parser.add_argument(
            "fake_args",
            metavar="fake argument",
            action="store",
            nargs="*",
            help="optional arguments to pass to the fake "
            "(e.g. the profile fake takes an optional "
            "list of comma separated field names as the "
            "first argument)",
        )

        arguments = parser.parse_args(self.argv[1:])
        if arguments.include is None:
            arguments.include = META_PROVIDERS_MODULES

        if arguments.verbose:
            logging.basicConfig(level=logging.DEBUG)
        else:
            logging.basicConfig(level=logging.CRITICAL)

        random.seed(arguments.seed)
        seeds = [random.random() for _ in range(arguments.repeat)]

        for i in range(arguments.repeat):
            print_doc(
                arguments.fake,
                arguments.fake_args,
                lang=arguments.lang,
                output=arguments.o,
                seed=seeds[i],
                includes=arguments.include,
            )
            print(arguments.sep, file=arguments.o)

            if not arguments.fake:
                # repeat not supported for all docs
                break


def execute_from_command_line(argv: Optional[str] = None) -> None:
    """A simple method that runs a Command."""
    if sys.stdout.encoding is None:
        print(
            "please set python env PYTHONIOENCODING=UTF-8, example: "
            "export PYTHONIOENCODING=UTF-8, when writing to stdout",
            file=sys.stderr,
        )
        exit(1)

    command = Command(argv)
    command.execute()


if __name__ == "__main__":
    execute_from_command_line()



from datetime import date
from string import ascii_uppercase
from typing import Optional

from faker.utils.checksums import luhn_checksum

from .. import Provider as BaseProvider


class Provider(BaseProvider):
    """
    A Faker provider for the German VAT ID and the pension insurance number

    Sources:

    - http://ec.europa.eu/taxation_customs/vies/faq.html#item_11
    - https://de.wikipedia.org/wiki/Versicherungsnummer
    """

    vat_id_formats = ("DE#########",)

    def __letter_to_digit_string(self, letter: str) -> str:
        digit = ascii_uppercase.index(letter) + 1
        if len(str(digit)) == 2:
            return str(digit)
        return "0" + str(digit)

    def __get_rvnr_checkdigit(self, rvnr: str) -> str:
        # replace the letter at index 8 with its corresponding number
        letter = rvnr[8]
        rvnr = rvnr[:8] + self.__letter_to_digit_string(letter) + rvnr[9:]

        # calculate the product of each digit with the corresponding factor
        factors = [2, 1, 2, 5, 7, 1, 2, 1, 2, 1, 2, 1]
        products = []
        for index, digit in enumerate(rvnr):
            products.append(int(digit) * factors[index])

        # calculate the digit sum for each product
        digit_sums = []
        for product in products:
            digit_sum = 0
            while product:
                digit_sum += product % 10
                product = product // 10
            digit_sums.append(digit_sum)

        # get the check digit by summing up the digit sums and calculating the modulo of 10
        return str(sum(digit_sums) % 10)

    def vat_id(self) -> str:
        """
        http://ec.europa.eu/taxation_customs/vies/faq.html#item_11

        :return: A random German VAT ID
        """

        return self.bothify(self.random_element(self.vat_id_formats))

    def rvnr(self, birthdate: Optional[date] = None) -> str:
        """
        Pension insurance number (German: "Rentenversicherungsnummer", abbr. "RVNR")

        Source: https://de.wikipedia.org/wiki/Versicherungsnummer

        :return: A valid German pension insurance number
        """
        _birthdate = birthdate or self.generator.date_object()
        format: str = f"##{_birthdate:%d%m%y}?##"
        rvnr: str = self.bothify(format, letters=ascii_uppercase)

        return rvnr + self.__get_rvnr_checkdigit(rvnr)

    def kvnr(self) -> str:
        """
        German health insurance number ("Krankenversichertennummer", abbr. "KVNR")

        Source: https://de.wikipedia.org/wiki/Krankenversichertennummer

        :return: a random health insurance number
        """

        letter_number: str = str(self.random_int(min=1, max=26))
        if len(letter_number) == 1:
            letter_number = "0" + letter_number

        first_part_format: str = letter_number + "########"
        first_part: str = self.numerify(first_part_format)
        first_checkdigit: int = luhn_checksum(int(first_part[::-1]))
        second_part_format: str = "#########"
        second_part: str = self.numerify(second_part_format)

        kvnr: str = first_part + str(first_checkdigit) + second_part
        kvnr_checkdigit: int = luhn_checksum(int(kvnr[::-1]))
        kvnr = kvnr + str(kvnr_checkdigit)

        letter: str = ascii_uppercase[int(letter_number) - 1]
        kvnr = letter + kvnr[2:]

        return kvnr



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    street_prefixes = (
        "Av",
        "Avenida",
        "R.",
        "Rua",
        "Travessa",
        "Largo",
        "Alameda",
        "Praça",
    )

    city_formats = ("{{city_name}}",)
    street_name_formats = (
        "{{street_prefix}} {{last_name}}",
        "{{street_prefix}} {{first_name}} {{last_name}}",
        "{{street_prefix}} de {{last_name}}",
        "{{street_prefix}} {{place_name}}",
    )

    street_address_formats = ("{{street_name}}, {{building_number}}",)

    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)

    building_number_formats = ("S/N", "%", "%#", "%#", "%#", "%##")

    postcode_formats = ("%###-###",)

    cities = (
        "Abrantes",
        "Agualva-Cacém",
        "Albufeira",
        "Alcobaça",
        "Alcácer do Sal",
        "Almada",
        "Almeirim",
        "Alverca do Ribatejo",
        "Amadora",
        "Amarante",
        "Amora",
        "Anadia",
        "Angra do Heroísmo",
        "Aveiro",
        "Barcelos",
        "Barreiro",
        "Beja",
        "Braga",
        "Bragança",
        "Caldas da Rainha",
        "Caniço",
        "Cantanhede",
        "Cartaxo",
        "Castelo Branco",
        "Chaves",
        "Coimbra",
        "Costa da Caparica",
        "Covilhã",
        "Câmara de Lobos",
        "Elvas",
        "Entroncamento",
        "Ermesinde",
        "Esmoriz",
        "Espinho",
        "Esposende",
        "Estarreja",
        "Estremoz",
        "Fafe",
        "Faro",
        "Felgueiras",
        "Figueira da Foz",
        "Fiães",
        "Freamunde",
        "Funchal",
        "Fundão",
        "Fátima",
        "Gafanha da Nazaré",
        "Gandra",
        "Gondomar",
        "Gouveia",
        "Guarda",
        "Guimarães",
        "Horta",
        "Lagoa",
        "Lagos",
        "Lamego",
        "Leiria",
        "Lisboa",
        "Lixa",
        "Loulé",
        "Loures",
        "Lourosa",
        "Macedo de Cavaleiros",
        "Maia",
        "Mangualde",
        "Marco de Canaveses",
        "Marinha Grande",
        "Matosinhos",
        "Mealhada",
        "Miranda do Douro",
        "Mirandela",
        "Montemor-o-Novo",
        "Montijo",
        "Moura",
        "Mêda",
        "Odivelas",
        "Olhão",
        "Oliveira de Azeméis",
        "Oliveira do Bairro",
        "Oliveira do Hospital",
        "Ourém",
        "Ovar",
        "Paredes",
        "Paços de Ferreira",
        "Penafiel",
        "Peniche",
        "Peso da Régua",
        "Pinhel",
        "Pombal",
        "Ponta Delgada",
        "Ponte de Sor",
        "Portalegre",
        "Portimão",
        "Porto",
        "Porto Santo",
        "Praia da Vitória",
        "Póvoa de Santa Iria",
        "Póvoa de Varzim",
        "Quarteira",
        "Queluz",
        "Rebordosa",
        "Reguengos de Monsaraz",
        "Ribeira Grande",
        "Rio Maior",
        "Rio Tinto",
        "Sabugal",
        "Sacavém",
        "Santa Comba Dão",
        "Santa Cruz",
        "Santa Maria da Feira",
        "Santana",
        "Santarém",
        "Santiago do Cacém",
        "Santo Tirso",
        "Seia",
        "Seixal",
        "Serpa",
        "Setúbal",
        "Silves",
        "Sines",
        "Sintra",
        "São João da Madeira",
        "São Mamede de Infesta",
        "São Salvador de Lordelo",
        "Tarouca",
        "Tavira",
        "Tomar",
        "Tondela",
        "Torres Novas",
        "Torres Vedras",
        "Trancoso",
        "Trofa",
        "Valbom",
        "Vale de Cambra",
        "Valongo",
        "Valpaços",
        "Vendas Novas",
        "Viana do Castelo",
        "Vila Franca de Xira",
        "Vila Nova de Famalicão",
        "Vila Nova de Foz Côa",
        "Vila Nova de Gaia",
        "Vila Nova de Santo André",
        "Vila Real",
        "Vila Real de Santo António",
        "Vila do Conde",
        "Viseu",
        "Vizela",
        "Évora",
        "Ílhavo",
    )

    countries = (
        "Afeganistão",
        "África do Sul",
        "Akrotiri",
        "Albânia",
        "Alemanha",
        "Andorra",
        "Angola",
        "Anguila",
        "Antárctida",
        "Antígua e Barbuda",
        "Antilhas Neerlandesas",
        "Arábia Saudita",
        "Arctic Ocean",
        "Argélia",
        "Argentina",
        "Arménia",
        "Aruba",
        "Ashmore and Cartier Islands",
        "Atlantic Ocean",
        "Austrália",
        "Áustria",
        "Azerbaijão",
        "Baamas",
        "Bangladeche",
        "Barbados",
        "Barém",
        "Bélgica",
        "Belize",
        "Benim",
        "Bermudas",
        "Bielorrússia",
        "Birmânia",
        "Bolívia",
        "Bósnia e Herzegovina",
        "Botsuana",
        "Brasil",
        "Brunei",
        "Bulgária",
        "Burquina Faso",
        "Burúndi",
        "Butão",
        "Cabo Verde",
        "Camarões",
        "Camboja",
        "Canadá",
        "Catar",
        "Cazaquistão",
        "Chade",
        "Chile",
        "China",
        "Chipre",
        "Clipperton Island",
        "Colômbia",
        "Comores",
        "Congo-Brazzaville",
        "Congo-Kinshasa",
        "Coral Sea Islands",
        "Coreia do Norte",
        "Coreia do Sul",
        "Costa do Marfim",
        "Costa Rica",
        "Croácia",
        "Cuba",
        "Dhekelia",
        "Dinamarca",
        "Domínica",
        "Egipto",
        "Emiratos Árabes Unidos",
        "Equador",
        "Eritreia",
        "Eslováquia",
        "Eslovénia",
        "Espanha",
        "Estados Unidos",
        "Estónia",
        "Etiópia",
        "Faroé",
        "Fiji",
        "Filipinas",
        "Finlândia",
        "França",
        "Gabão",
        "Gâmbia",
        "Gana",
        "Gaza Strip",
        "Geórgia",
        "Geórgia do Sul e Sandwich do Sul",
        "Gibraltar",
        "Granada",
        "Grécia",
        "Gronelândia",
        "Guame",
        "Guatemala",
        "Guernsey",
        "Guiana",
        "Guiné",
        "Guiné Equatorial",
        "Guiné-Bissau",
        "Haiti",
        "Honduras",
        "Hong Kong",
        "Hungria",
        "Iémen",
        "Ilha Bouvet",
        "Ilha do Natal",
        "Ilha Norfolk",
        "Ilhas Caimão",
        "Ilhas Cook",
        "Ilhas dos Cocos",
        "Ilhas Falkland",
        "Ilhas Heard e McDonald",
        "Ilhas Marshall",
        "Ilhas Salomão",
        "Ilhas Turcas e Caicos",
        "Ilhas Virgens Americanas",
        "Ilhas Virgens Britânicas",
        "Índia",
        "Indian Ocean",
        "Indonésia",
        "Irão",
        "Iraque",
        "Irlanda",
        "Islândia",
        "Israel",
        "Itália",
        "Jamaica",
        "Jan Mayen",
        "Japão",
        "Jersey",
        "Jibuti",
        "Jordânia",
        "Kuwait",
        "Laos",
        "Lesoto",
        "Letónia",
        "Líbano",
        "Libéria",
        "Líbia",
        "Listenstaine",
        "Lituânia",
        "Luxemburgo",
        "Macau",
        "Macedónia do Norte",
        "Madagáscar",
        "Malásia",
        "Malávi",
        "Maldivas",
        "Mali",
        "Malta",
        "Man, Isle of",
        "Marianas do Norte",
        "Marrocos",
        "Maurícia",
        "Mauritânia",
        "Mayotte",
        "México",
        "Micronésia",
        "Moçambique",
        "Moldávia",
        "Mónaco",
        "Mongólia",
        "Monserrate",
        "Montenegro",
        "Mundo",
        "Namíbia",
        "Nauru",
        "Navassa Island",
        "Nepal",
        "Nicarágua",
        "Níger",
        "Nigéria",
        "Niue",
        "Noruega",
        "Nova Caledónia",
        "Nova Zelândia",
        "Omã",
        "Pacific Ocean",
        "Países Baixos",
        "Palau",
        "Panamá",
        "Papua-Nova Guiné",
        "Paquistão",
        "Paracel Islands",
        "Paraguai",
        "Peru",
        "Pitcairn",
        "Polinésia Francesa",
        "Polónia",
        "Porto Rico",
        "Portugal",
        "Quénia",
        "Quirguizistão",
        "Quiribáti",
        "Reino Unido",
        "República Centro-Africana",
        "República Checa",
        "República Dominicana",
        "Roménia",
        "Ruanda",
        "Rússia",
        "Salvador",
        "Samoa",
        "Samoa Americana",
        "Santa Helena",
        "Santa Lúcia",
        "São Cristóvão e Neves",
        "São Marinho",
        "São Pedro e Miquelon",
        "São Tomé e Príncipe",
        "São Vicente e Granadinas",
        "Sara Ocidental",
        "Seicheles",
        "Senegal",
        "Serra Leoa",
        "Sérvia",
        "Singapura",
        "Síria",
        "Somália",
        "Southern Ocean",
        "Spratly Islands",
        "Sri Lanca",
        "Suazilândia",
        "Sudão",
        "Suécia",
        "Suíça",
        "Suriname",
        "Svalbard e Jan Mayen",
        "Tailândia",
        "Taiwan",
        "Tajiquistão",
        "Tanzânia",
        "Território Britânico do Oceano Índico",
        "Territórios Austrais Franceses",
        "Timor Leste",
        "Togo",
        "Tokelau",
        "Tonga",
        "Trindade e Tobago",
        "Tunísia",
        "Turquemenistão",
        "Turquia",
        "Tuvalu",
        "Ucrânia",
        "Uganda",
        "União Europeia",
        "Uruguai",
        "Usbequistão",
        "Vanuatu",
        "Vaticano",
        "Venezuela",
        "Vietname",
        "Wake Island",
        "Wallis e Futuna",
        "West Bank",
        "Zâmbia",
        "Zimbabué",
    )

    # From https://pt.wikipedia.org/wiki/Distritos_de_Portugal
    distritos = (
        "Aveiro",
        "Beja",
        "Braga",
        "Bragança",
        "Castelo Branco",
        "Coimbra",
        "Évora",
        "Faro",
        "Guarda",
        "Leiria",
        "Lisboa",
        "Portalegre",
        "Porto",
        "Santarém",
        "Setúbal",
        "Viana do Castelo",
        "Vila Real",
        "Viseu",
    )

    # From https://pt.wikipedia.org/wiki/Lista_de_concelhos_por_NUTS,_distritos_e_ilhas
    concelhos = (
        "Águeda",
        "Aguiar da Beira",
        "Alandroal",
        "Albergaria-a-Velha",
        "Albufeira",
        "Alcácer do Sal",
        "Alcanena",
        "Alcobaça",
        "Alcochete",
        "Alcoutim",
        "Alenquer",
        "Alfândega da Fé",
        "Alijó",
        "Aljezur",
        "Aljustrel",
        "Almada",
        "Almeida",
        "Almeirim",
        "Almodôvar",
        "Alpiarça",
        "Alter do Chão",
        "Alvaiázere",
        "Alvito",
        "Amadora",
        "Amarante",
        "Amares",
        "Anadia",
        "Angra do Heroísmo",
        "Ansião",
        "Arcos de Valdevez",
        "Arganil",
        "Armamar",
        "Arouca",
        "Arraiolos",
        "Arronches",
        "Arruda dos Vinhos",
        "Aveiro",
        "Avis",
        "Azambuja",
        "Baião",
        "Barcelos",
        "Barrancos",
        "Barreiro",
        "Batalha",
        "Beja",
        "Belmonte",
        "Benavente",
        "Bombarral",
        "Borba",
        "Boticas",
        "Braga",
        "Bragança",
        "Cabeceiras de Basto",
        "Cadaval",
        "Caldas da Rainha",
        "Calheta (R.A.A.)",
        "Calheta (R.A.M.)",
        "Câmara de Lobos",
        "Caminha",
        "Campo Maior",
        "Cantanhede",
        "Carrazeda de Ansiães",
        "Carregal do Sal",
        "Cartaxo",
        "Cascais",
        "Castanheira de Pêra",
        "Castelo Branco",
        "Castelo de Paiva",
        "Castelo de Vide",
        "Castro Daire",
        "Castro Marim",
        "Castro Verde",
        "Celorico da Beira",
        "Celorico de Basto",
        "Chamusca",
        "Chaves",
        "Cinfães",
        "Coimbra",
        "Condeixa-a-Nova",
        "Constância",
        "Coruche",
        "Corvo",
        "Covilhã",
        "Crato",
        "Cuba",
        "Elvas",
        "Entroncamento",
        "Espinho",
        "Esposende",
        "Estarreja",
        "Estremoz",
        "Évora",
        "Fafe",
        "Faro",
        "Felgueiras",
        "Ferreira do Alentejo",
        "Ferreira do Zêzere",
        "Figueira da Foz",
        "Figueira de Castelo Rodrigo",
        "Figueiró dos Vinhos",
        "Fornos de Algodres",
        "Freixo de Espada à Cinta",
        "Fronteira",
        "Funchal",
        "Fundão",
        "Gavião",
        "Góis",
        "Golegã",
        "Gondomar",
        "Gouveia",
        "Grândola",
        "Guarda",
        "Guimarães",
        "Horta",
        "Idanha-a-Nova",
        "Ílhavo",
        "Lagoa",
        "Lagoa (R.A.A)",
        "Lagos",
        "Lajes das Flores",
        "Lajes do Pico",
        "Lamego",
        "Leiria",
        "Lisboa",
        "Loulé",
        "Loures",
        "Lourinhã",
        "Lousã",
        "Lousada",
        "Mação",
        "Macedo de Cavaleiros",
        "Machico",
        "Madalena",
        "Mafra",
        "Maia",
        "Mangualde",
        "Manteigas",
        "Marco de Canaveses",
        "Marinha Grande",
        "Marvão",
        "Matosinhos",
        "Mealhada",
        "Meda",
        "Melgaço",
        "Mértola",
        "Mesão Frio",
        "Mira",
        "Miranda do Corvo",
        "Miranda do Douro",
        "Mirandela",
        "Mogadouro",
        "Moimenta da Beira",
        "Moita",
        "Monção",
        "Monchique",
        "Mondim de Basto",
        "Monforte",
        "Montalegre",
        "Montemor-o-Novo",
        "Montemor-o-Velho",
        "Montijo",
        "Mora",
        "Mortágua",
        "Moura",
        "Mourão",
        "Murça",
        "Murtosa",
        "Nazaré",
        "Nelas",
        "Nisa",
        "Nordeste",
        "Óbidos",
        "Odemira",
        "Odivelas",
        "Oeiras",
        "Oleiros",
        "Olhão",
        "Oliveira de Azeméis",
        "Oliveira de Frades",
        "Oliveira do Bairro",
        "Oliveira do Hospital",
        "Ourém",
        "Ourique",
        "Ovar",
        "Paços de Ferreira",
        "Palmela",
        "Pampilhosa da Serra",
        "Paredes",
        "Paredes de Coura",
        "Pedrógão Grande",
        "Penacova",
        "Penafiel",
        "Penalva do Castelo",
        "Penamacor",
        "Penedono",
        "Penela",
        "Peniche",
        "Peso da Régua",
        "Pinhel",
        "Pombal",
        "Ponta Delgada",
        "Ponta do Sol",
        "Ponte da Barca",
        "Ponte de Lima",
        "Ponte de Sor",
        "Portalegre",
        "Portel",
        "Portimão",
        "Porto",
        "Porto de Mós",
        "Porto Moniz",
        "Porto Santo",
        "Povoação",
        "Póvoa de Lanhoso",
        "Póvoa de Varzim",
        "Proença-a-Nova",
        "Redondo",
        "Reguengos de Monsaraz",
        "Resende",
        "Ribeira Brava",
        "Ribeira de Pena",
        "Ribeira Grande",
        "Rio Maior",
        "Sabrosa",
        "Sabugal",
        "Salvaterra de Magos",
        "Santa Comba Dão",
        "Santa Cruz",
        "Santa Cruz da Graciosa",
        "Santa Cruz das Flores",
        "Santa Maria da Feira",
        "Santa Marta de Penaguião",
        "Santana",
        "Santarém",
        "Santiago do Cacém",
        "Santo Tirso",
        "São Brás de Alportel",
        "São João da Madeira",
        "São João da Pesqueira",
        "São Pedro do Sul",
        "São Roque do Pico",
        "São Vicente",
        "Sardoal",
        "Sátão",
        "Seia",
        "Seixal",
        "Sernancelhe",
        "Serpa",
        "Sertã",
        "Sesimbra",
        "Setúbal",
        "Sever do Vouga",
        "Silves",
        "Sines",
        "Sintra",
        "Sobral de Monte Agraço",
        "Soure",
        "Sousel",
        "Tábua",
        "Tabuaço",
        "Tarouca",
        "Tavira",
        "Terras de Bouro",
        "Tomar",
        "Tondela",
        "Torre de Moncorvo",
        "Torres Novas",
        "Torres Vedras",
        "Trancoso",
        "Trofa",
        "Vagos",
        "Vale de Cambra",
        "Valença",
        "Valongo",
        "Valpaços",
        "Velas",
        "Vendas Novas",
        "Viana do Alentejo",
        "Viana do Castelo",
        "Vidigueira",
        "Vieira do Minho",
        "Vila da Praia da Vitória",
        "Vila de Rei",
        "Vila do Bispo",
        "Vila do Conde",
        "Vila do Porto",
        "Vila Flor",
        "Vila Franca de Xira",
        "Vila Franca do Campo",
        "Vila Nova da Barquinha",
        "Vila Nova de Cerveira",
        "Vila Nova de Famalicão",
        "Vila Nova de Foz Côa",
        "Vila Nova de Gaia",
        "Vila Nova de Paiva",
        "Vila Nova de Poiares",
        "Vila Pouca de Aguiar",
        "Vila Real",
        "Vila Real de Santo António",
        "Vila Velha de Ródão",
        "Vila Verde",
        "Vila Viçosa",
        "Vimioso",
        "Vinhais",
        "Viseu",
        "Vizela",
        "Vouzela",
    )

    # From https://pt.wikipedia.org/wiki/Lista_de_freguesias_de_Portugal
    freguesias = [
        "Abrantes",
        "Águeda",
        "Aguiar da Beira",
        "Alandroal",
        "Albergaria-a-Velha",
        "Albufeira",
        "Alcácer do Sal",
        "Alcanena",
        "Alcobaça",
        "Alcochete",
        "Alcoutim",
        "Alenquer",
        "Alfândega da Fé",
        "Alijó",
        "Aljezur",
        "Aljustrel",
        "Almada",
        "Almeida",
        "Almeirim",
        "Almodôvar",
        "Alpiarça",
        "Alter do Chão",
        "Alvaiázere",
        "Alvito",
        "Amadora",
        "Amarante",
        "Amares",
        "Anadia",
        "Angra do Heroísmo",
        "Ansião",
        "Arcos de Valdevez",
        "Arganil",
        "Armamar",
        "Arouca",
        "Arraiolos",
        "Arronches",
        "Arruda dos Vinhos",
        "Aveiro",
        "Avis",
        "Azambuja",
        "Baião",
        "Barcelos",
        "Barrancos",
        "Barreiro",
        "Batalha",
        "Beja",
        "Belmonte",
        "Benavente",
        "Bombarral",
        "Borba",
        "Boticas",
        "Braga",
        "Bragança",
        "Cabeceiras de Basto",
        "Cadaval",
        "Caldas da Rainha",
        "Calheta (Açores)",
        "Calheta (Madeira)",
        "Câmara de Lobos",
        "Caminha",
        "Campo Maior",
        "Cantanhede",
        "Carrazeda de Ansiães",
        "Carregal do Sal",
        "Cartaxo",
        "Cascais",
        "Castanheira de Pêra",
        "Castelo Branco",
        "Castelo de Paiva",
        "Castelo de Vide",
        "Castro Daire",
        "Castro Marim",
        "Castro Verde",
        "Celorico da Beira",
        "Celorico de Basto",
        "Chamusca",
        "Chaves",
        "Cinfães",
        "Coimbra",
        "Condeixa-a-Nova",
        "Constância",
        "Coruche",
        "Corvo",
        "Covilhã",
        "Crato",
        "Cuba",
        "Elvas",
        "Entroncamento",
        "Espinho",
        "Esposende",
        "Estarreja",
        "Estremoz",
        "Évora",
        "Fafe",
        "Faro",
        "Felgueiras",
        "Ferreira do Alentejo",
        "Ferreira do Zêzere",
        "Figueira da Foz",
        "Figueira de Castelo Rodrigo",
        "Figueiró dos Vinhos",
        "Fornos de Algodres",
        "Freixo de Espada à Cinta",
        "Fronteira",
        "Funchal",
        "Fundão",
        "Gavião",
        "Góis",
        "Golegã",
        "Gondomar",
        "Gouveia",
        "Grândola",
        "Guarda",
        "Guimarães",
        "Horta",
        "Idanha-a-Nova",
        "Ílhavo",
        "Lagoa",
        "Lagoa (Açores)",
        "Lagos",
        "Lajes das Flores",
        "Lajes do Pico",
        "Lamego",
        "Leiria",
        "Lisboa",
        "Loulé",
        "Loures",
        "Lourinhã",
        "Lousã",
        "Lousada",
        "Mação",
        "Macedo de Cavaleiros",
        "Machico",
        "Madalena",
        "Mafra",
        "Maia",
        "Mangualde",
        "Manteigas",
        "Marco de Canaveses",
        "Marinha Grande",
        "Marvão",
        "Matosinhos",
        "Mealhada",
        "Mêda",
        "Melgaço",
        "Mértola",
        "Mesão Frio",
        "Mira",
        "Miranda do Corvo",
        "Miranda do Douro",
        "Mirandela",
        "Mogadouro",
        "Moimenta da Beira",
        "Moita",
        "Monção",
        "Monchique",
        "Mondim de Basto",
        "Monforte",
        "Montalegre",
        "Montemor-o-Novo",
        "Montemor-o-Velho",
        "Montijo",
        "Mora",
        "Mortágua",
        "Moura",
        "Mourão",
        "Murça",
        "Murtosa",
        "Nazaré",
        "Nelas",
        "Nisa",
        "Nordeste",
        "Óbidos",
        "Odemira",
        "Odivelas",
        "Oeiras",
        "Oleiros",
        "Olhão",
        "Oliveira de Azeméis",
        "Oliveira de Frades",
        "Oliveira do Bairro",
        "Oliveira do Hospital",
        "Ourém",
        "Ourique",
        "Ovar",
        "Paços de Ferreira",
        "Palmela",
        "Pampilhosa da Serra",
        "Paredes",
        "Paredes de Coura",
        "Pedrógão Grande",
        "Penacova",
        "Penafiel",
        "Penalva do Castelo",
        "Penamacor",
        "Penedono",
        "Penela",
        "Peniche",
        "Peso da Régua",
        "Pinhel",
        "Pombal",
        "Ponta Delgada",
        "Ponta do Sol",
        "Ponte da Barca",
        "Ponte de Lima",
        "Ponte de Sor",
        "Portalegre",
        "Portel",
        "Portimão",
        "Porto",
        "Porto de Mós",
        "Porto Moniz",
        "Porto Santo",
        "Póvoa de Lanhoso",
        "Póvoa de Varzim",
        "Povoação",
        "Praia da Vitória",
        "Proença-a-Nova",
        "Redondo",
        "Reguengos de Monsaraz",
        "Resende",
        "Ribeira Brava",
        "Ribeira de Pena",
        "Ribeira Grande",
        "Rio Maior",
        "Sabrosa",
        "Sabugal",
        "Salvaterra de Magos",
        "Santa Comba Dão",
        "Santa Cruz",
        "Santa Cruz da Graciosa",
        "Santa Cruz das Flores",
        "Santa Maria da Feira",
        "Santa Marta de Penaguião",
        "Santana",
        "Santarém",
        "Santiago do Cacém",
        "Santo Tirso",
        "São Brás de Alportel",
        "São João da Madeira",
        "São João da Pesqueira",
        "São Pedro do Sul",
        "São Roque do Pico",
        "São Vicente (Madeira)",
        "Sardoal",
        "Sátão",
        "Seia",
        "Seixal",
        "Sernancelhe",
        "Serpa",
        "Sertã",
        "Sesimbra",
        "Setúbal",
        "Sever do Vouga",
        "Silves",
        "Sines",
        "Sintra",
        "Sobral de Monte Agraço",
        "Soure",
        "Sousel",
        "Tábua",
        "Tabuaço",
        "Tarouca",
        "Tavira",
        "Terras de Bouro",
        "Tomar",
        "Tondela",
        "Torre de Moncorvo",
        "Torres Novas",
        "Torres Vedras",
        "Trancoso",
        "Trofa",
        "Vagos",
        "Vale de Cambra",
        "Valença",
        "Valongo",
        "Valpaços",
        "Velas",
        "Vendas Novas",
        "Viana do Alentejo",
        "Viana do Castelo",
        "Vidigueira",
        "Vieira do Minho",
        "Vila de Rei",
        "Vila do Bispo",
        "Vila do Conde",
        "Vila do Porto",
        "Vila Flor",
        "Vila Franca de Xira",
        "Vila Franca do Campo",
        "Vila Nova da Barquinha",
        "Vila Nova de Cerveira",
        "Vila Nova de Famalicão",
        "Vila Nova de Foz Côa",
        "Vila Nova de Gaia",
        "Vila Nova de Paiva",
        "Vila Nova de Poiares",
        "Vila Pouca de Aguiar",
        "Vila Real",
        "Vila Real de Santo António",
        "Vila Velha de Ródão",
        "Vila Verde",
        "Vila Viçosa",
        "Vimioso",
        "Vinhais",
        "Viseu",
        "Vizela",
        "Vouzela",
    ]

    # from https://pt.wikipedia.org/wiki/Lista_de_arruamentos_de_Lisboa
    # and https://pt.wikipedia.org/wiki/Lista_de_arruamentos_do_Porto
    places = (
        "da Igreja",
        "António Sérgio",
        "Cardeal Cerejeira",
        "Coronel Marques Júnior",
        "da Encarnação",
        "da Música",
        "da Quinta de Santo António",
        "da Universidade",
        "das Comunidades Portuguesas",
        "das Linhas de Torres",
        "de Santo António dos Capuchos",
        "do Beato",
        "Dom Afonso Henriques",
        "dos Oceanos",
        "dos Pinheiros",
        "Edgar Cardoso",
        "Mahatma Gandhi",
        "Manuel Ricardo Espírito Santo",
        "Padre Álvaro Proença",
        "Roentgen",
        "da Boavista",
        "da Cova da Moura",
        "das Conchas",
        "de Caselas",
        "de São Francisco",
        "do Carvalhão",
        "do Longo",
        "do Penalva",
        "do Varejão",
        "dos Moinhos",
        "da Conceição",
        "das Portas do Mar",
        "de Jesus",
        "do Evaristo",
        "do Rosário",
        "Escuro",
        "Grande de Cima",
        "Areeiro",
        "Campolide",
        "Madrid",
        "Paris (Nascente)",
        "Paris (Poente)",
        "Roma",
        "Sabugosa",
        "Novo (à Travessa das Águas Boas)",
        "da Ponte da Lama",
        "da Praia da Galé",
        "do Duro",
        "dos Ferreiros",
        "das Rolas",
        "da Lingueta",
        "das Naus",
        "do Olival",
        "do Sodré",
        "dos Argonautas",
        "Português",
        "da Figueira",
        "de Santo Estêvão",
        "de São Lourenço",
        "de São Miguel",
        "do Tijolo",
        "dos Olivais",
        "da Feiteira",
        "da Rainha",
        "da Raposa",
        "das Andorinhas",
        "das Cegonhas",
        "das Gaivotas ao Parque das Nações",
        "de Baixo da Penha",
        "de Palma de Cima",
        "do Alto do Varejão",
        "do Arboreto",
        "dos Estorninhos",
        "dos Flamingos",
        "dos Melros",
        "dos Pardais",
        "dos Pinheiros ao Parque das Nações",
        "dos Rouxinóis",
        "Velho do Outeiro",
        "das Amoreiras",
        "das Cebolas",
        "de Santa Clara",
        "dos Mártires da Pátria",
        "Grande",
        "Pequeno",
        "de Campolide",
        "da Graça",
        "de Colares",
        "Norte do Bairro da Encarnação",
        "Sul do Bairro da Encarnação",
        "da Torrinha",
        "do Castelo",
        "de Santa Helena",
        "da Sé",
        "das Bolas",
        "das Chagas",
        "José António Marques",
        "do Monte",
        "Gerais",
        "D. Carlos I ao Parque das Nações",
        "Adão Barata",
        "Alfredo Keil",
        "Alice Cruz",
        "Amália Rodrigues",
        "Amélia Carvalheira",
        "Amnistia Internacional",
        "Augusto Monjardino",
        "Bento Martins",
        "das Nações",
        "Ducla Soares",
        "Eduardo Prado Coelho",
        "Elisa Baptista de Sousa Pedroso",
        "Fernanda de Castro",
        "Fernando Pessa",
        "Ferreira de Mira",
        "Garcia de Orta ao Parque das Nações",
        "Irmã Lúcia",
        "Jorge Luis Borges",
        "Luís Ferreira",
        "Maria da Luz Ponces de Carvalho",
        "Maria de Lourdes Sá Teixeira",
        "Maria José Moura",
        "Mário Ruivo",
        "Mário Soares",
        "9 de Abril",
        "Prof. António de Sousa Franco",
        "Prof. Francisco Caldeira Cabral",
        "Pulido Garcia",
        "Tristão da Silva",
        "Ribeirinhos",
        "Sophia de Mello Breyner Andresen",
        "do Mirante",
        "do Alto de São João",
        "General Afonso Botelho",
        "Eduardo VII de Inglaterra",
        "Silva Porto",
        "Artur Agostinho",
        "da Ilha dos Amores",
        "da Nau Catrineta",
        "da Vila Expo",
        "das Âncoras",
        "das Fragatas",
        "das Garças",
        "das Gáveas ao Parque das Nações",
        "das Musas",
        "das Tágides",
        "de Neptuno",
        "de Ulisses",
        "do Adamastor",
        "do Amazonas",
        "do Báltico",
        "do Campo da Bola",
        "do Cantábrico",
        "do Levante",
        "do Parque",
        "do Ródano",
        "do Sapal",
        "do Tejo",
        "do Trancão",
        "dos Aventureiros",
        "dos Cruzados",
        "dos Fenícios",
        "dos Heróis do Mar",
        "dos Jacarandás",
        "dos Mastros",
        "dos Navegadores",
        "João Jayme Faria Affonso",
        "Júlio Verne",
        "Afonso de Albuquerque",
        "da Cruz",
        "da Galega",
        "das Canas",
        "das Galeotas ao Parque das Nações",
        "das Pirogas",
        "de Dom Fradique",
        "do Carrasco",
        "do Peneireiro",
        "do Pimenta",
        "do Pinzaleiro",
        "do Seabra",
        "do Sequeiro",
        "do Sextante",
        "do Tronco",
        "dos Escaleres",
        "do Borratém",
        "do Mar",
        "Adolfo Ayala",
        "Cuf",
        "da Quinta de São João Baptista",
        "da Quinta do Guarda-Mor",
        "da Rua Duque de Palmela",
        "das Torres do Restelo",
        "do Chinquilho",
        "Fernando Valle",
        "Maestro Ivo Cruz",
        "Prof. António José Saraiva",
        "Professor Gonçalves Ferreira",
        "Professor José Conde",
        "Teófilo Ferreira",
        "das Necessidades",
        "do Mercado",
        "dos Anjos",
        "do Conde de Óbidos",
        "de Palma",
        "Almirante Pinheiro de Azevedo",
        "António Dias Lourenço",
        "Coronel Vítor Alves",
        "da Expo 98",
        "das Olaias",
        "das Oliveiras",
        "de Pina Manique",
        "dos Vice-reis",
        "Matilde Bensaúde",
        "Nelson Mandela",
        "Pupilos do Exército",
        "República Argentina",
        "República da Colômbia",
        "Visconde de Alvalade",
        "do Barcal",
        "do Calhau",
        "de São Vicente",
        "das Ondas",
        "dos Corvos",
        "Feia",
        "Arquitecto Carlos Ramos",
        "das Antas",
        "das Fontainhas",
        "de 25 de Abril",
        "de Aquilino Ribeiro",
        "de Basílio Teles",
        "de Cartes",
        "de Cláudio Carneiro",
        "de Eça de Queirós",
        "de Manuel d'Arriaga",
        "do Dr. António Macedo",
        "do Dr. Fernando de Azeredo Antas",
        "do Prof. Hernâni Monteiro",
        "do Prof. Ruy Luís Gomes",
        "dos Capitães de Abril",
        "25 de Abril",
        "da Associação Empresarial de Portugal",
        "da França",
        "de Camilo",
        "de D. Afonso Henriques",
        "de D. Carlos I",
        "de D. João II",
        "de Fernão de Magalhães",
        "de Fontes Pereira de Melo",
        "de Gustavo Eiffel",
        "de Montevideu",
        "de Nun'Álvares Pereira",
        "de Paiva Couceiro",
        "de Rodrigues de Freitas",
        "de Sidónio Pais",
        "de Vasco da Gama",
        "de Vímara Peres",
        "do Bessa",
        "do Brasil (Porto)",
        "do Conselho da Europa",
        "do Dr. Antunes Guimarães",
        "do Marechal Gomes da Costa",
        "dos Aliados",
        "dos Combatentes da Grande Guerra",
        "Flor da Rosa",
        "José Domingues dos Santos",
        "da Agra do Amial",
        "da Fonte da Moura",
        "da Pasteleira",
        "da Rainha D. Leonor",
        "de Costa Cabral",
        "de Francos",
        "de Manuel Cardoso Agrelos",
        "de Pio XII",
        "de Ramalde",
        "de São João de Deus",
        "de São Roque da Lameira",
        "de São Vicente de Paulo",
        "de Santo Eugénio",
        "do Aleixo",
        "do Bom Sucesso",
        "do Carvalhido",
        "do Cerco do Porto",
        "do Dr. Nuno Pinheiro Torres",
        "do Falcão",
        "do Lagarteiro",
        "do Leal",
        "do Outeiro",
        "do Regado",
        "do Viso",
        "Herculano",
        "Central",
        "da Bela Vista",
        "da Beneditina",
        "da Senhora da Luz",
        "de Bonjóia",
        "de Carreiras",
        "de Passos Manuel",
        "de S. João da Foz",
        "de S. Macário",
        "de S. Marçal",
        "do Arrabalde",
        "do Campo",
        "do Campo Alegre",
        "do Machado",
        "do Meiral",
        "do Paço",
        "do Pedregulho",
        "do Preto",
        "de Baixo",
        "de Cima",
        "da Alfândega",
        "da Estiva",
        "da Ribeira",
        "das Pedras",
        "do Bicalho",
        "dos Guindais",
        "da Arrábida",
        "da Boa Viagem",
        "da Póvoa",
        "da Ranha",
        "das Carquejeiras",
        "das Laranjeiras",
        "das Virtudes",
        "de Chaves de Oliveira",
        "de D. Pedro Pitões",
        "de Godim",
        "de João do Carmo",
        "de Maceda",
        "de Marques Marinho",
        "de Monchique",
        "de Nova Sintra",
        "de São Pedro",
        "de Serrúbia",
        "de Sobre-o-Douro",
        "de Vandoma",
        "do Calvário",
        "do Carregal",
        "do Forno Velho",
        "do Monte da Lapa",
        "do Monte de S. João",
        "do Ouro",
        "do Rego Lameiro",
        "dos Ingleses",
        "da Fonte de Cima",
        "das Congostas",
        "da Asprela",
        "de Vinte e Quatro de Agosto",
        "do Rou",
        "de Antero de Quental",
        "de Estêvão Vasconcelos",
        "de Viterbo de Campos",
        "do Dr. Manuel Laranjeira",
        "Carolina Michaelis de Vasconcelos",
        "da Vitória",
        "das Sereias",
        "das Verdades",
        "de S. Francisco de Borja",
        "do Adro",
        "do Barredo",
        "do Caminho Novo",
        "do Cidral de Baixo",
        "do Cidral de Cima",
        "do Codeçal",
        "do Colégio",
        "do Monte Cativo",
        "do Monte dos Judeus",
        "do Pinheiro",
        "do Recanto",
        "do Roleto",
        "dos Armazéns",
        "do Molhe",
        "da Circunvalação",
        "de Gondomar",
        "Nacional 108",
        "Nacional 209",
        "de Moradias Populares do Eng.º Machado Vaz",
        "de Moradais Populares do Carriçal",
        "de Antero de Figueiredo",
        "de Arnaldo Gama",
        "de Belém",
        "de Carrilho Videira",
        "de Guedes de Oliveira",
        "de João Chagas",
        "de Marques de Oliveira",
        "de Teófilo Braga",
        "do Moreda",
        "do Passeio Alegre",
        "Machado de Asis",
        "Severo Portela",
        "da Foz",
        "do Bolhão",
        "dos Bacalhoeiros",
        "da Luz",
        "do Seminário",
        "S. Bartolomeu",
        "de S. Lázaro",
        "das Escadas do Monte dos Judeus",
        "das Japoneiras",
        "de S. Salvador",
        "do Bonjardim",
        "de Luiz I",
        "de Maria Pia",
        "do Freixo",
        "do Carvão",
        "da Banda de Ramalde",
        "da Cidade da Praia",
        "das Mimosas",
        "de Adelino Amaro da Costa",
        "de Augusto Gomes",
        "de Bernarda Ferreira Lacerda",
        "de Eduardo Soares",
        "de Francisco Borges",
        "de Irene de Castro",
        "de João Augusto Ribeiro",
        "de José Régio",
        "de José Serra",
        "de Luís António Verney",
        "de Públia Hortênsia",
        "de Ribeiro Sanches",
        "de S. Mamede",
        "do Dr. Jaime Cortesão",
        "do Maestro Afonso Valentim",
        "do Maestro Resende Dias",
        "do Mestre de Aviz",
        "do Prof. Egas Moniz",
        "Egito Gonçalves",
        "Ernesto Veiga de Oliveira",
        "João Glama",
        "José Luís Nunes",
        "Manuel Gonçalves Moreira",
        "Artur Cupertino de Miranda",
        "Associação Empresarial de Portugal",
        "Manuel Pinto de Azevedo Júnior",
        "Goelas de Pau",
        "de Cintura Interna",
        "do Almirante Gago Coutinho",
        "do Castelo do Queijo",
        "Futebol Clube do Porto",
        "Panorâmica",
        "Panorâmica Edgar Cardoso",
        "de Gonçalo Cristóvão",
        "do Cais das Pedras",
        "da Aldeia",
        "da Baleia",
        "da Bouça",
        "da Carvalhosa",
        "da Companhia",
        "da Ilha do Ferro",
        "da Pedreira",
        "da Senhora da Lapa",
        "das Andrezas",
        "de Grijó",
        "de Lamas",
        "de S. Brás",
        "de Santana",
        "do Anjo",
        "do Anjo da Guarda",
        "do Buraco",
        "do José da Mestra",
        "do Monte da Pena",
        "do Picoto",
        "do Sobreirinho",
    )

    def street_prefix(self) -> str:
        """
        :example: 'Rua'
        """
        return self.random_element(self.street_prefixes)

    def city_name(self) -> str:
        """
        :example: 'Amora'
        """
        return self.random_element(self.cities)

    def administrative_unit(self) -> str:
        """
        :example: 'Bragança'
        """
        return self.random_element(self.distritos)

    distrito = administrative_unit

    def concelho(self) -> str:
        """
        :example: 'Tondela'
        """
        return self.random_element(self.concelhos)

    def freguesia(self) -> str:
        """
        :example: 'Miranda do Douro'
        """
        return self.random_element(self.freguesias)

    def place_name(self) -> str:
        """
        :example: "do Pombal"
        """
        return self.random_element(self.places)



import re

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``th_TH`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Thailand
    """

    license_formats = (
        "# ?? ####",
        "# ?? ###",
        "# ?? ##",
        "# ?? #",
        "?? ####",
        "?? ###",
        "?? ##",
        "?? #",
        "??? ###",
        "??? ##",
        "??? #",
        "##-####",
    )

    thai_consonants = "กขฃคฅฆงจฉชซฌญฎฏฐฑฒณดตถทธนบปผฝพฟภมยรลวศษสหฬอฮ"

    def license_plate(self) -> str:
        """Generate a license plate."""

        temp = re.sub(
            r"\?",
            lambda x: self.random_element(self.thai_consonants),
            self.random_element(self.license_formats),
        )
        return self.numerify(temp)



from .. import Provider as PersonProvider


class Provider(PersonProvider):

    formats_male = (
        "{{first_name_male}} {{last_name}}",
        "{{prefix_male}} {{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}{{suffix}}",
    )

    formats_female = (
        "{{first_name_female}} {{last_name}}",
        "{{prefix_female}} {{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}{{suffix}}",
    )

    formats = (
        "{{first_name}} {{last_name}}",
        "{{prefix}} {{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}{{suffix}}",
    )

    # Source references (accessed 2026-04-18):
    # - https://www.behindthename.com/names/usage/marathi (given names)
    # - https://en.wikipedia.org/wiki/Category:Marathi-language_surnames (surnames)
    # Data is manually filtered for synthetic test-data suitability.

    first_names_female = (
        "अंजली",
        "अनिता",
        "अनुष्का",
        "अपर्णा",
        "आकांक्षा",
        "आशा",
        "इशिता",
        "उमा",
        "कविता",
        "कल्पना",
        "कीर्ती",
        "कोमल",
        "गौरी",
        "चेतना",
        "ज्योती",
        "तन्वी",
        "दीपाली",
        "नेहा",
        "पल्लवी",
        "पूजा",
        "प्रज्ञा",
        "प्रिया",
        "माधुरी",
        "मानसी",
        "मीनल",
        "मेघा",
        "रेणुका",
        "रीमा",
        "रोहिणी",
        "लता",
        "वैशाली",
        "शिल्पा",
        "श्रुती",
        "स्मिता",
        "स्वाती",
        "स्नेहा",
        "हर्षदा",
    )

    first_names_male = (
        "अभिजीत",
        "अभिषेक",
        "अमित",
        "अमोल",
        "अनिल",
        "अनिरुद्ध",
        "अरुण",
        "अशोक",
        "आदित्य",
        "आकाश",
        "ईश्वर",
        "उमेश",
        "किरण",
        "कैलास",
        "गणेश",
        "गौरव",
        "चेतन",
        "जितेंद्र",
        "दत्तात्रेय",
        "दीपक",
        "धनंजय",
        "निखिल",
        "निलेश",
        "पंकज",
        "प्रकाश",
        "प्रवीण",
        "प्रशांत",
        "भूषण",
        "मकरंद",
        "मंगेश",
        "माधव",
        "मिलिंद",
        "मोहन",
        "राहुल",
        "रोहित",
        "विकास",
        "विनायक",
        "विवेक",
        "सचिन",
        "संदीप",
        "समीर",
        "सुनील",
        "सुरेश",
        "हर्षद",
    )

    first_names = first_names_female + first_names_male

    last_names = (
        "आगरकर",
        "अहेर",
        "ओक",
        "कदम",
        "कांबळे",
        "कुलकर्णी",
        "कोल्हे",
        "गवळी",
        "गायकवाड",
        "चव्हाण",
        "चौगुले",
        "जगताप",
        "जाधव",
        "जोशी",
        "ठाकूर",
        "देशमुख",
        "देशपांडे",
        "पाटील",
        "पवार",
        "फडके",
        "भोसले",
        "महाजन",
        "माने",
        "मोरे",
        "राऊत",
        "शिंदे",
        "शिरसाट",
        "साळुंखे",
        "साने",
        "सावंत",
    )

    prefixes_female = (
        "श्रीमती",
        "कुमारी",
    )

    prefixes_male = ("श्री",)

    prefixes = prefixes_female + prefixes_male

    suffixes = ("जी",)



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    city_formats = ("{{city_name}}",)

    street_name_formats = (
        "{{first_name}} {{last_name}}",
        "{{last_name}}",
    )

    street_address_formats = ("{{building_number}} {{street_name}}",)

    address_formats = (
        "{{street_address}}\n{{city}} {{postcode}}",
        "{{street_address}}\n{{city}}-{{postcode}}",
    )

    building_number_formats = (
        "####",
        "###",
        "##",
        "#",
        "#/#",
        "##/##",
        "##/###",
        "##/####",
    )

    postcode_formats = ("######",)

    # Source references (accessed 2026-04-18):
    # - https://en.wikipedia.org/wiki/List_of_cities_in_Maharashtra_by_population (cities)
    # - https://www.india.gov.in/my-government/states-uts (states baseline)
    # - https://github.com/unicode-org/cldr-json/blob/main/cldr-json/cldr-localenames-full/main/mr/territories.json
    #   (Marathi territory/country names)
    # Data is manually filtered and normalized for Faker synthetic generation.

    cities = (
        "अहमदनगर",
        "अकोला",
        "अमरावती",
        "औरंगाबाद",
        "उस्मानाबाद",
        "कोल्हापूर",
        "गोंदिया",
        "चंद्रपूर",
        "जळगाव",
        "ठाणे",
        "धुळे",
        "नवी मुंबई",
        "नागपूर",
        "नांदेड",
        "नाशिक",
        "पिंपरी-चिंचवड",
        "पुणे",
        "परभणी",
        "बीड",
        "मुंबई",
        "मिरज",
        "यवतमाळ",
        "रत्नागिरी",
        "लातूर",
        "वसई",
        "सांगली",
        "सातारा",
        "सोलापूर",
        "हिंगोली",
    )

    states = (
        "आंध्र प्रदेश",
        "अरुणाचल प्रदेश",
        "आसाम",
        "उत्तर प्रदेश",
        "उत्तराखंड",
        "ओडिशा",
        "कर्नाटक",
        "केरळ",
        "गोवा",
        "गुजरात",
        "छत्तीसगड",
        "झारखंड",
        "तामिळनाडू",
        "तेलंगणा",
        "त्रिपुरा",
        "नागालँड",
        "पंजाब",
        "पश्चिम बंगाल",
        "बिहार",
        "मणिपूर",
        "मध्य प्रदेश",
        "महाराष्ट्र",
        "मेघालय",
        "मिझोरम",
        "राजस्थान",
        "सिक्कीम",
        "हरियाणा",
        "हिमाचल प्रदेश",
    )

    countries = (
        "ऑस्ट्रेलिया",
        "कॅनडा",
        "जर्मनी",
        "जपान",
        "नेपाळ",
        "पाकिस्तान",
        "भारत",
        "भूतान",
        "बांग्लादेश",
        "फ्रान्स",
        "युनायटेड किंगडम",
        "संयुक्त अरब अमिरात",
        "संयुक्त राज्य अमेरिका",
        "सिंगापूर",
        "श्रीलंका",
    )

    def city_name(self) -> str:
        return self.random_element(self.cities)

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit



import random

from .. import Provider as BaseProvider


class Provider(BaseProvider):
    """
    A Faker provider for the Spanish VAT IDs and DOIs
    """

    vat_id_formats = (
        "ES?########",
        "ES########?",
        "ES?#######?",
    )

    def vat_id(self) -> str:
        """
        http://ec.europa.eu/taxation_customs/vies/faq.html#item_11
        :return: a random Spanish VAT ID

        :sample:
        """

        return self.bothify(self.random_element(self.vat_id_formats))

    def nie(self) -> str:
        """
        https://es.wikipedia.org/wiki/N%C3%BAmero_de_identidad_de_extranjero
        :return: a random Spanish NIE

        :sample:
        """

        first_chr = random.randrange(0, 3)
        doi_body = str(random.randrange(0, 10000000)).zfill(7)
        control = self._calculate_control_doi(str(first_chr) + doi_body)
        return "XYZ"[first_chr] + doi_body + control

    def nif(self) -> str:
        """
        https://es.wikipedia.org/wiki/N%C3%BAmero_de_identificaci%C3%B3n_fiscal
        :return: NIF

        :sample:
        """

        nie_body = str(random.randrange(0, 100000000))  # generate a number of a maximum of 8 characters long
        return nie_body.zfill(8) + self._calculate_control_doi(nie_body)

    def cif(self) -> str:
        """
        https://es.wikipedia.org/wiki/C%C3%B3digo_de_identificaci%C3%B3n_fiscal
        :return: a random Spanish CIF

        :sample:
        """

        first_chr = random.choice("ABCDEFGHJNPQRSUVW")
        doi_body = str(random.randrange(0, 10000000)).zfill(7)
        cif = first_chr + doi_body
        return cif + self._calculate_control_cif(cif)

    def nuss(self, company: bool = False) -> str:
        """
        :param company: flag to indicate if we should generate a company NUSS
        :return: a random Spanish Social Security Number (Número de la Seguridad Social)

        :sample:
        :sample: company=True
        """
        nuss_body_length = 8
        if company:
            nuss_body_length = 7
        province_digits = f"{random.choice(list(range(1, 54)) + [66]):02d}"
        nuss_body = "".join(str(random.randint(0, 9)) for _ in range(nuss_body_length))
        control_digits = f"{int(province_digits+nuss_body) % 97:02d}"
        nuss = f"{province_digits}{nuss_body}{control_digits}"
        return nuss

    @staticmethod
    def _calculate_control_doi(doi: str) -> str:
        """
        Calculate the letter that corresponds to the end of a DOI
        :param doi: calculated value so far needing a control character
        :return: DOI control character
        """

        lookup = "TRWAGMYFPDXBNJZSQVHLCKE"
        return lookup[int(doi) % 23]

    @classmethod
    def _calculate_control_cif(cls, cif: str) -> str:
        """
        Calculate the letter that corresponds to the end of a CIF
        :param cif: calculated value so far needing a control character
        :return: CIF control character

        Code was converted from the minified js of: https://generadordni.es/
        """

        sum_ = 0
        first_chr, cif_value = cif[0], cif[1:]
        for index, char in enumerate(cif_value):
            if index % 2:
                sum_ += int(char)
            else:
                sum_ += sum(map(int, str(int(char) * 2)))
        if sum_ > 10:
            sum_ = int(str(sum_)[-1])
        else:
            sum_ = sum_
        sum_ = 10 - (sum_ % 10)

        if first_chr in ["F", "J", "K", "N", "P", "Q", "R", "S", "U", "V", "W"]:
            return chr(64 + sum_)
        elif first_chr in ["A", "B", "C", "D", "E", "F", "G", "H", "L", "M"]:
            if sum_ == 10:
                sum_ = 0
            return str(sum_)
        else:  # K, L, M  # pragma: no cover
            # Old format that is no longer used, here for full compatability
            return cls._calculate_control_doi(cif)  # pragma: no cover



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    """Provider for datetimes for fil_PH locale"""

    DAY_NAMES = {
        "0": "Linggo",
        "1": "Lunes",
        "2": "Martes",
        "3": "Miyerkules",
        "4": "Huwebes",
        "5": "Biyernes",
        "6": "Sabado",
    }
    MONTH_NAMES = {
        "01": "Enero",
        "02": "Pebrero",
        "03": "Marso",
        "04": "Abril",
        "05": "Mayo",
        "06": "Hunyo",
        "07": "Hulyo",
        "08": "Agosto",
        "09": "Setyembre",
        "10": "Oktubre",
        "11": "Nobyembre",
        "12": "Disyembre",
    }

    def day_of_week(self):
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        return self.MONTH_NAMES[month]



from .. import Provider as JobProvider


class Provider(JobProvider):
    """Translated from Super class"""

    jobs = (
        # Information technology field
        "Lập trình viên",
        "Kỹ sư phần mềm",
        "Kiến trúc sư phần mềm",
        "Nhà phân tích dữ liệu",
        "Chuyên viên bảo mật",
        "Tester",
        "DevOps Engineer",
        "Project Manager",
        "UX/UI Designer",
        "Digital Marketer",
        "Thực Tập",
        # Finance - banking sector
        "Nhân viên ngân hàng",
        "Chuyên viên tín dụng",
        "Kế toán",
        "Kiểm toán",
        "Nhà tư vấn tài chính",
        "Chuyên viên phân tích thị trường",
        # Business areas
        "Giám đốc kinh doanh",
        "Trưởng phòng kinh doanh",
        "Nhân viên kinh doanh",
        "Marketing Manager",
        "Sales Representative",
        "Chuyên viên bán hàng trực tuyến",
        # Education Department
        "Giáo viên",
        "Giảng viên",
        "Chuyên viên tư vấn tuyển sinh",
        "Thực tập sinh giáo dục",
        # Medical
        "Bác sĩ",
        "Y tá",
        "Dược sĩ",
        "Điều Dưỡng",
        # Building sector
        "Kỹ sư xây dựng",
        "Kiến trúc sư",
        "Thợ xây",
        "Kỹ sư giám sát",
        # Service sector
        "Nhân viên khách sạn",
        "Nhân viên nhà hàng",
        "Tư vấn khách hàng",
        "Nhân viên lễ tân",
        # Manufacturing sector
        "Công nhân sản xuất",
        "Kỹ sư sản xuất",
        "Quản lý sản xuất",
        # Agriculture sector
        "Nông dân",
        "Kỹ sư nông nghiệp",
        # Law field
        "Luật sư",
        "Thư ký pháp lý",
        # Other areas
        "Nhà báo",
        "Biên dịch viên",
        "Nghệ sĩ",
        "Nhà thiết kế đồ họa",
        "Nhân viên hành chính",
        "Chuyên viên nhân sự",
        "Nhân Viên Bán Hàng",
    )

    def job(self) -> str:
        return self.random_element(self.jobs)



from .. import Provider as BaseProvider


class Provider(BaseProvider):

    # Sources:
    #   - https://www.ons.dz/IMG/file/Profession.pdf
    jobs_male = (
        "أستاذ",
        "أخصائي اجتماعي",
        "إداري",
        "إطار",
        "بائع",
        "تاجر",
        "تقني",
        "تقني سامٍ",
        "جراح",
        "حارس أمن",
        "خبير اقتصادي",
        "خبير محاسب",
        "سائق",
        "شرطي",
        "صحفي",
        "صيدلي",
        "طبيب",
        "طبيب أسنان",
        "طبيب بيطري",
        "طيار",
        "عامل",
        "فلاح",
        "قاضي",
        "كاتب عمومي",
        "مبرمج",
        "مترجم",
        "محامي",
        "محاسب",
        "محرر",
        "مدرب",
        "مدير",
        "مدير مشروع",
        "مرشد سياحي",
        "مستشار قانوني",
        "مصمم",
        "مصمم جرافيك",
        "مصور",
        "معلم",
        "مفتش",
        "مقاول",
        "ممرض",
        "ممثل تجاري",
        "موظف إداري",
        "موظف بنكي",
        "مهندس",
        "مهندس بترولي",
        "مهندس معماري",
        "ميكانيكي",
        "نجار",
        "وكيل عقاري",
    )

    jobs_female = (
        "أستاذة",
        "أخصائية اجتماعية",
        "إدارية",
        "إطارة",
        "بائعة",
        "تاجرة",
        "تقنية",
        "تقنية سامية",
        "جراحة",
        "حارسة أمن",
        "خبيرة اقتصادية",
        "خبيرة محاسبة",
        "سائقة",
        "شرطية",
        "صحفية",
        "صيدلانية",
        "طبيبة",
        "طبيبة أسنان",
        "طبيبة بيطرية",
        "طيارة",
        "عاملة",
        "فلاحة",
        "قاضية",
        "كاتبة عمومية",
        "مبرمجة",
        "مترجمة",
        "محامية",
        "محاسبة",
        "محررة",
        "مدربة",
        "مديرة",
        "مديرة مشروع",
        "مرشدة سياحية",
        "مستشارة قانونية",
        "مصممة",
        "مصممة جرافيك",
        "مصورة",
        "معلمة",
        "مفتشة",
        "مقاولة",
        "ممرضة",
        "ممثلة تجارية",
        "موظفة إدارية",
        "موظفة بنكية",
        "مهندسة",
        "مهندسة بترولية",
        "مهندسة معمارية",
        "ميكانيكية",
        "وكيلة عقارية",
    )

    jobs = jobs_male + jobs_female



import string

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``de_AT`` locale.

    Sources:

    - https://de.wikipedia.org/wiki/Kfz-Kennzeichen_(%C3%96sterreich)
    """

    license_plate_prefix = (
        "A",
        "AM",
        "B",
        "BA",
        "BB",
        "BD",
        "BG",
        "BH",
        "BK",
        "BL",
        "BM",
        "BN",
        "BP",
        "BR",
        "BZ",
        "DL",
        "DO",
        "E",
        "EF",
        "EU",
        "FB",
        "FE",
        "FF",
        "FK",
        "FR",
        "FV",
        "FW",
        "G",
        "GB",
        "GD",
        "GF",
        "GK",
        "GM",
        "GR",
        "GS",
        "GU",
        "HA",
        "HB",
        "HE",
        "HF",
        "HL",
        "HO",
        "I",
        "IL",
        "IM",
        "JE",
        "JO",
        "JU",
        "JW",
        "K",
        "KB",
        "KD",
        "KF",
        "KG",
        "KI",
        "KK",
        "KL",
        "KO",
        "KR",
        "KS",
        "KU",
        "L",
        "LA",
        "LB",
        "LD",
        "LE",
        "LF",
        "LI",
        "LK",
        "LL",
        "LN",
        "LZ",
        "MA",
        "MD",
        "ME",
        "MI",
        "MT",
        "MU",
        "MZ",
        "N",
        "ND",
        "NK",
        "O",
        "OP",
        "OW",
        "P",
        "PE",
        "PL",
        "PT",
        "RA",
        "RE",
        "RI",
        "RO",
        "S",
        "SB",
        "SD",
        "SE",
        "SK",
        "SL",
        "SO",
        "SP",
        "SR",
        "ST",
        "SV",
        "SW",
        "SZ",
        "T",
        "TA",
        "TD",
        "TK",
        "TU",
        "UU",
        "V",
        "VB",
        "VD",
        "VI",
        "VK",
        "VL",
        "VO",
        "W",
        "WB",
        "WD",
        "WE",
        "WK",
        "WL",
        "WN",
        "WO",
        "WT",
        "WU",
        "WY",
        "WZ",
        "ZE",
        "ZT",
        "ZW",
    )

    license_plate_suffix_for_one_starting_letter = ("-%# ???", "-%## ???", "-%## ??", "-%### ??", "-%### ?", "-%#### ?")

    license_plate_suffix_for_two_starting_letters = (
        "-% ???",
        "-%# ???",
        "-%# ??",
        "-%## ??",
        "-%## ?",
        "-%### ?",
    )

    def license_plate(self) -> str:
        """Generate a license plate."""
        prefix: str = self.random_element(self.license_plate_prefix)

        if len(prefix) == 1:
            suffix = self.bothify(
                self.random_element(self.license_plate_suffix_for_one_starting_letter),
                letters=string.ascii_uppercase,
            )
        else:
            suffix = self.bothify(
                self.random_element(self.license_plate_suffix_for_two_starting_letters),
                letters=string.ascii_uppercase,
            )

        return prefix + suffix



import logging

from faker.providers.bank import Provider as BankProvider

logger = logging.getLogger(__name__)


class Provider(BankProvider):
    """Implement bank provider for ``en_PH`` locale."""

    country_code = "PH"
    bban_format = "################"
    swift_bank_codes = (
        "ANZB",
        "AUBK",
        "BKCH",
        "BKKB",
        "BNOR",
        "BNPA",
        "BOFA",
        "BOPI",
        "BOTK",
        "BPDI",
        "BPFS",
        "BPGO",
        "CHAS",
        "CHBK",
        "CHSV",
        "CITI",
        "CPHI",
        "CTCB",
        "DBPH",
        "DEUT",
        "EQSN",
        "EWBC",
        "FCBK",
        "HBPH",
        "HNBK",
        "HSBC",
        "IBKO",
        "ICBC",
        "INGB",
        "KOEX",
        "MBBE",
        "MBTC",
        "MHCB",
        "PABI",
        "PHSB",
        "PHTB",
        "PHVB",
        "PNBM",
        "PPBU",
        "RCBC",
        "ROBP",
        "SCBL",
        "SETC",
        "SHBK",
        "SMBC",
        "STLA",
        "TACB",
        "TLBP",
        "TYBK",
        "UBPH",
        "UCPB",
        "UOVB",
        "UWCB",
    )
    swift_location_codes = (
        "22",
        "2X",
        "M1",
        "MM",
        "MQ",
        "MX",
    )
    swift_branch_codes = (
        "CBU",
        "EQI",
        "TSU",
        "XXX",
    )

    def bban(self) -> str:
        """Generate a Basic Bank Account Number (BBAN).

        .. warning::
           Philippine bank accounts do not have BBANs or IBANs, so any number
           generated by this method is a purely hypothetical number. Local bank
           account numbers are typically 10 or 12 digits long, so the BBAN
           format used in this implementation has been arbitrarily set to 16
           digits to simulate a hypothetical standardization of account numbers.
           Using this method will log a warning regarding the hypotheticality of
           the result.
        """
        logger.warning("Numbers generated by this method are purely hypothetical.")
        return super().bban()

    def iban(self) -> str:
        """Generate an International Bank Account Number (IBAN).

        .. warning::
           Philippine bank accounts do not have BBANs or IBANs, so any number
           generated by this method is a purely hypothetical number. This method
           uses hypothetical PH BBANs and the PH country code as inputs to the
           IBAN generation algorithm. Using this method will log a warning
           regarding the hypotheticality of the result.
        """
        logger.warning("Numbers generated by this method are purely hypothetical.")
        return super().iban()



from .. import Provider as SsnProvider


class Provider(SsnProvider):
    """
    Sources:

    - https://fr.wikipedia.org/wiki/Num%C3%A9ro_d%27identification_national_(Alg%C3%A9rie)
    - https://github.com/itshakim213/dz-nin-checker
    """

    def _control_key(self, base: str) -> str:
        """Compute the 2-digit control key via modified Luhn over 16 digits."""
        total, alternate = 0, False
        for i in range(len(base) - 1, -1, -1):
            d = int(base[i]) * (2 if alternate else 1)
            total += d - 9 if d > 9 else d
            alternate = not alternate
        remainder = total % 10
        return f"{(0 if remainder == 0 else 10 - remainder):02d}"

    def ssn(self) -> str:
        """Generate an Algerian National Identification Number (NIN).

        Structure (18 digits):

        - 1 digit  – nationality: ``1`` = Algerian, ``2`` = dual nationality
        - 1 digit  – sex: ``0`` = male, ``1`` = female
        - 3 digits – last three digits of the birth-registration year
        - 4 digits – commune code (wilaya 01–58 + commune 01–20)
        - 5 digits – birth-certificate act number
        - 2 digits – annual register serial number
        - 2 digits – control key (modified Luhn)
        """
        nationality = self.random_element(("1", "2"))
        sex = self.random_element(("0", "1"))
        year_code = f"{self.random_int(min=1950, max=2006) % 1000:03d}"
        wilaya = self.random_int(min=1, max=58)
        commune_code = f"{wilaya:02d}{self.random_int(min=1, max=20):02d}"
        act_code = f"{self.random_int(min=1, max=99999):05d}"
        register_code = f"{self.random_int(min=1, max=99):02d}"
        base = nationality + sex + year_code + commune_code + act_code + register_code
        return base + self._control_key(base)



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Automotive provider for mk_MK locale (Macedonian).

    Sources:
    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_North_Macedonia

    Standard format: XX NNNN YY
    where XX = two-letter regional code, NNNN = 4-digit number, YY = two letters

    Notes:
    - Letters Q, W, X, Y are NOT used (no Cyrillic equivalents).
    - Country identifier changed from MK to NMK in February 2019 (Prespa Agreement).
    """

    # All 34 official regional codes as of 2020
    # Sources: original 1993 codes + expansions in 2012, 2013, 2015, 2019, 2020
    license_plate_prefixes = (
        "BE",  # Berovo
        "BT",  # Bitola
        "DB",  # Debar
        "DE",  # Delčevo
        "DH",  # Demir Hisar
        "DK",  # Demir Kapija
        "GE",  # Gevgelija
        "GV",  # Gostivar
        "KA",  # Kavadarci
        "KI",  # Kičevo
        "KO",  # Kočani
        "KP",  # Kriva Palanka
        "KR",  # Kratovo
        "KS",  # Kruševo
        "KU",  # Kumanovo
        "MB",  # Makedonski Brod
        "MK",  # Makedonska Kamenica (municipality)
        "NE",  # Negotino
        "OH",  # Ohrid
        "PE",  # Pehčevo
        "PP",  # Prilep
        "PS",  # Probištip
        "RA",  # Radoviš
        "RE",  # Resen
        "SK",  # Skopje
        "SN",  # Sveti Nikole
        "SR",  # Strumica
        "ST",  # Štip
        "SU",  # Struga
        "TE",  # Tetovo
        "VA",  # Valandovo
        "VE",  # Veles
        "VI",  # Vinica
        "VV",  # Vevčani
    )

    # Latin letters used on plates — Q, W, X, Y are excluded (no Cyrillic equivalents)
    license_plate_suffix_letters = "ABCDEFGHIJKLMNOPRSTUVZ"

    def license_plate(self) -> str:
        prefix = self.random_element(self.license_plate_prefixes)
        number = self.numerify("####")
        suffix = self.random_element(self.license_plate_suffix_letters) + self.random_element(
            self.license_plate_suffix_letters
        )
        return f"{prefix} {number} {suffix}"



from collections import OrderedDict

from .. import Provider as ColorProvider


class Provider(ColorProvider):
    """Implement color provider for ``hy_AM`` locale."""

    all_colors = OrderedDict(
        (
            ("Ալիսի կապույտ", "#F0F8FF"),
            ("Անանուխի կրեմ", "#F5FFFA"),
            ("Անտառային կանաչ", "#228B22"),
            ("Արծաթագույն", "#C0C0C0"),
            ("Արքայական կապույտ", "#4169E1"),
            ("Բաց դեղին", "#FFFFE0"),
            ("Բաց դեղնաոսկեգույն", "#FAFAD2"),
            ("Բաց երկնագույն", "#87CEFA"),
            ("Բաց ծովային կանաչ", "#20B2AA"),
            ("Բաց կաթնագույն", "#FFFFF0"),
            ("Բաց կանաչ", "#90EE90"),
            ("Բաց կապույտ", "#ADD8E6"),
            ("Բաց կապտականաչ", "#E0FFFF"),
            ("Բաց կորալ", "#F08080"),
            ("Բաց մանուշակագույն", "#EE82EE"),
            ("Բաց մոխրագույն թերթաքար", "#778899"),
            ("Բաց մոխրագույն", "#D3D3D3"),
            ("Բաց նշագույն", "#FFEBCD"),
            ("Բաց պողպատե կապույտ", "#B0C4DE"),
            ("Բաց սաղմոնագույն", "#FFA07A"),
            ("Բաց վարդագույն", "#FFB6C1"),
            ("Բեժ", "#F5F5DC"),
            ("Բոսորագույն", "#DC143C"),
            ("Գարնանային կանաչ", "#00FF7F"),
            ("Գեյնսբորրո", "#DCDCDC"),
            ("Գունատ կանաչ", "#98FB98"),
            ("Գունատ կարմիր մանուշակագույն", "#DB7093"),
            ("Գունատ ոսկեգույն", "#EEE8AA"),
            ("Գունատ փիրուզագույն", "#AFEEEE"),
            ("Գրասենյակային կանաչ", "#008000"),
            ("Դարչնագույն ավազ", "#F4A460"),
            ("Դարչնագույն", "#964B00"),
            ("Դեղին", "#FFFF00"),
            ("Դեղձի կրեմ", "#FFDAB9"),
            ("Դեղնականաչ", "#9ACD3"),
            ("Դոդջերս կապույտ", "#1E90FF"),
            ("Եգիպտացորենի մազիկներ", "#FFF8DC"),
            ("Երկնագույն մառախուղ", "#F0FFFF"),
            ("Երկնագույն", "#87CEEB"),
            ("Զինվորական կանաչ", "#6B8E23"),
            ("Թամբի դարչնագույն", "#8B4513"),
            ("Թեժ վարդագույն", "#FF69B4"),
            ("Թուխ", "#D2B48C"),
            ("Ինդիգո", "#4B0082"),
            ("Լայմի կանաչ", "#32CD32"),
            ("Լավանդ", "#E6E6FA"),
            ("Լոլիկ", "#FF6347"),
            ("Խակի", "#F0E68C"),
            ("Խոլորձագույն", "#DA70D6"),
            ("Ծխագույն", "#F5F5F5"),
            ("Ծովախեցի", "#FFF5EE"),
            ("Ծովակնագույն", "#7FFFD4"),
            ("Ծովային կանաչ", "#2E8B57"),
            ("Կադետների կապույտ", "#5F9EA0"),
            ("Կաթնագույն", "#FFFAF0"),
            ("Կակաոյի դարչնագույն", "#D2691E"),
            ("Կանաչ", "#00FF00"),
            ("Կանաչադեղին", "#ADFF2F"),
            ("Կապույտ թերթաքար", "#6A5ACD"),
            ("Կապույտ մանուշակագույն", "#8A2BE2"),
            ("Կապույտ փոշի", "#B0E0E6"),
            ("Կապույտ", "#0000FF"),
            ("Կապտականաչ", "#00FFFF"),
            ("Կարմիր դարչնագույն", "#A52A2A"),
            ("Կարմիր լավանդ", "#FFF0F5"),
            ("Կարմիր մանուշակագույն", "#C71585"),
            ("Կարմիր", "#FF0000"),
            ("Կեսգիշերային կապույտ", "#191970"),
            ("Կիտրոնի շիֆոն", "#FFFACD"),
            ("Կորալ", "#FF7F50"),
            ("Հարած պապայա", "#FFEFD5"),
            ("Հին ժանյակ", "#FDF5E6"),
            ("Հնաոճ սպիտակ", "#FAEBD7"),
            ("Հնդկական կարմիր", "#CD5C5C"),
            ("Հրակայուն աղյուս", "#B22222"),
            ("Ձիթապտղի գույն", "#808000"),
            ("Ձյունաճերմակ", "#FFFAFA"),
            ("Մանուշակագույն", "#800080"),
            ("Մեղրացող սեխ", "#F0FFF0"),
            ("Միջին գարնանային կանաչ", "#00FA9A"),
            ("Միջին խոլորձագույն", "#BA55D3"),
            ("Միջին ծովակնագույն", "#66CDAA"),
            ("Միջին ծովային կանաչ", "#3CB371"),
            ("Միջին կապույտ թերթաքար", "#7B68EE"),
            ("Միջին կապույտ", "#0000CD"),
            ("Միջին կապտականաչ", "#9370DB"),
            ("Միջին փիրուզագույն", "#48D1CC"),
            ("Մոխրագույն թերթաքար", "#708090"),
            ("Մոխրագույն", "#808080"),
            ("Մոկասին", "#FFE4B5"),
            ("Մուգ երկնագույն", "#00BFFF"),
            ("Մուգ խակի", "#BDB76B"),
            ("Մուգ խոլորձագույն", "#9932CC"),
            ("Մուգ ծովային կանաչ", "#8FBC8F"),
            ("Մուգ կանաչ", "#006400"),
            ("Մուգ կապույտ թերթաքար", "#483D8B"),
            ("Մուգ կապույտ", "#00008B"),
            ("Մուգ կապտականաչ", "#008080"),
            ("Մուգ կարմիր", "#8B0000"),
            ("Մուգ ձիթապտղի կանաչ", "#556B2F"),
            ("Մուգ մանուշակագույն", "#9400D3"),
            ("Մուգ մոխրագույն թերթաքար", "#2F4F4F"),
            ("Մուգ մոխրագույն", "#696969"),
            ("Մուգ մոխրագույն", "#A9A9A9"),
            ("Մուգ նարնջագույն", "#FF8C00"),
            ("Մուգ ոսկեգույն", "#B8860B"),
            ("Մուգ սաղմոնագույն", "#E9967A"),
            ("Մուգ վառ մանուշակագույն", "#8B008B"),
            ("Մուգ վարդագույն", "#FF1493"),
            ("Մուգ փիրուզագույն", "#00CED1"),
            ("Նավահո սպիտակ", "#FFDEAD"),
            ("Նավատորմի կապույտ", "#000080"),
            ("Նարնջագույն կարմիր", "#FF4500"),
            ("Նարնջագույն", "#FFA500"),
            ("Նշագույն", "#FFE4C4"),
            ("Շագանակագույն", "#800000"),
            ("Շարտրուզ", "#7FFF00"),
            ("Ոսկեգույն ձող", "#DAA520"),
            ("Ոսկեգույն", "#FFD700"),
            ("Պերու", "#CD853F"),
            ("Պողպատե կապույտ", "#4682B4"),
            ("Սալոր", "#DDA0DD"),
            ("Սաղմոնագույն", "#FA8072"),
            ("Սիենա", "#A0522D"),
            ("Սիզամարգի կանաչ", "#7CFC00"),
            ("Սպիտակ ստվեր", "#F8F8FF"),
            ("Սպիտակ", "#FFFFFF"),
            ("Սև", "#000000"),
            ("Վառ մանուշակագույն", "#FF00FF"),
            ("Վարդագույն", "#FFC0CB"),
            ("Վարդագույն", "#FFE4E1"),
            ("Վարդադարչնագույն", "#BC8F8F"),
            ("Վուշ", "#FAF0E6"),
            ("Տատասկ", "#D8BFD8"),
            ("Տերեփուկի կապույտ", "#6495ED"),
            ("Ցորենագույն", "#F5DEB3"),
            ("Փիրուզագույն", "#40E0D0"),
            ("Փխրուն փայտ", "#DEB887"),
        )
    )

    safe_colors = (
        "սև",
        "շագանակագույն",
        "կանաչ",
        "նավատորմի կապույտ",
        "ձիթապտղի գույն",
        "մանուշակագույն",
        "մուգ կապտականաչ",
        "լայմ",
        "կապույտ",
        "արծաթագույն",
        "մոխրագույն",
        "դեղին",
        "վառ մանուշակագույն",
        "կապտականաչ",
        "սպիտակ",
    )



from decimal import Decimal

from .. import Provider as GeoProvider


class Provider(GeoProvider):
    """Geo provider for mk_MK locale (Macedonian).

    Sources:
    - https://en.wikipedia.org/wiki/Geography_of_North_Macedonia
      Bounding box: lat 40.85°N – 42.37°N, lon 20.45°E – 23.03°E
      Center (Skopje): lat 41.9981°N, lon 21.4254°E
    """

    def local_latitude(self) -> Decimal:
        """Return a latitude within North Macedonia's bounding box."""
        return self.coordinate(center=41.6, radius=0.75)

    def local_longitude(self) -> Decimal:
        """Return a longitude within North Macedonia's bounding box."""
        return self.coordinate(center=21.74, radius=1.29)



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    # Source (https://tr.wikipedia.org/wiki/T.C._Kimlik_Numaras%C4%B1)
    # Turkey Republic National Number (TCKN) is an 11-digit identity number.
    # Rules:
    # 1. The number consists of 11 digits.
    # 2. The first digit cannot be zero.
    # 3. The 10th digit is calculated as:
    #    ((sum of digits in odd positions (1st, 3rd, 5th, 7th, 9th) * 7)
    #    - sum of digits in even positions (2nd, 4th, 6th, 8th)) mod 10.
    # 4. The 11th digit is the modulo 10 of the sum of the first 10 digits.
    # 5. The number must satisfy these rules to be considered valid.

    def ssn(self) -> str:
        """
        :example: '85420031070'
        """
        digits = [self.random_int(1, 9)]

        for _ in range(8):
            digits.append(self.random_int(0, 9))

        odd_sum = sum(digits[i] for i in [0, 2, 4, 6, 8])
        even_sum = sum(digits[i] for i in [1, 3, 5, 7])
        tenth = ((odd_sum * 7) - even_sum) % 10
        digits.append(tenth)

        total_sum = sum(digits)
        eleventh = total_sum % 10
        digits.append(eleventh)

        return "".join(str(d) for d in digits)



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    """Company provider for ar_DZ locale (Algeria, Arabic-language)."""

    # Sources:
    #   - https://www.commerce.gov.dz/fr/choix-de-la-forme-juridique-de-votre-entreprise-1
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "مجمع {{last_name}}",
        "{{last_name}}",
    )

    company_suffixes = (
        "ش.ذ.م.م",
        "ش.م.",
        "م.ذ.م.م",
        "ش.ت.",
        "ش.ت.ب.",
        "م.ع.",
        "م.ع.ا.ت.",
    )

    catch_phrase_formats = ("{{catch_phrase_noun}} {{catch_phrase_verb}} {{catch_phrase_attribute}}",)

    nouns = (
        "الجودة",
        "التميّز",
        "الثقة",
        "التطوير",
        "النمو",
        "الابتكار",
        "الأداء",
        "التقدّم",
        "النجاح",
        "الخدمة",
        "الأمان",
        "الشراكة",
    )

    verbs = (
        "للنجاح",
        "للتطوير",
        "للتقدّم",
        "للابتكار",
        "للبناء",
        "للاستثمار",
        "لتحقيق أهدافك",
        "لإنجاز مشاريعك",
        "للنهوض بالاقتصاد",
        "لخدمة الوطن",
    )

    attributes = (
        "معاً",
        "بكفاءة",
        "بثقة",
        "بامتياز",
        "باستدامة",
        "في خدمة الجزائر",
        "نحو المستقبل",
        "من أجل غدٍ أفضل",
        "في قلب التنمية",
        "بجودة عالية",
    )

    def catch_phrase(self) -> str:
        pattern: str = self.random_element(self.catch_phrase_formats)
        return self.generator.parse(pattern)

    def catch_phrase_noun(self) -> str:
        return self.random_element(self.nouns)

    def catch_phrase_verb(self) -> str:
        return self.random_element(self.verbs)

    def catch_phrase_attribute(self) -> str:
        return self.random_element(self.attributes)



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    # Sourse: https://www.ganmarteba.ge/
    jobs = [
        "ავიადისპეტჩერი",
        "ამწე ოპერატორი",
        "ავიატექნიკოსი",
        "ავიაციის ტექნიკოსი",
        "აეროპლანის მფრინავი",
        "აკუსტიკის ინჟინერი",
        "აღმზრდელი",
        "აღმოსავლეთმცოდნე",
        "ავტოელექტრიკოსი",
        "აგრონომი",
        "არტისტი",
        "აგრონომ-ნიადაგმცოდნე",
        "ადვოკატი",
        "აკუმულატორების სპეციალისტი",
        "აქტუარი",
        "ალერგოლოგი",
        "ანალიტიკოსი",
        "ანდროლოგი",
        "ანთროპოლოგი",
        "არტილერისტი",
        "არქეოლოგი",
        "არქივარიუსი",
        "არქიტექტორი",
        "ასტრონომი",
        "ასტрофიზიკოსი",
        "ასტროქიმიკოსი",
        "ბაქტერიოლოგი",
        "ბალერინა",
        "ბალეტმაისტერი",
        "ბანკირი",
        "ბარმენი",
        "ბატალერი",
        "ბიბლიოთეკარი",
        "ბიბლიოთეკმცოდნე",
        "ბიონჟინერი",
        "ბიოლოგი",
        "ბიოფიზიკოსი",
        "ბიოქიმიკოსი",
        "ბლოგერი",
        "ბორტ-ინჟინერი",
        "ბორტ-მექანიკოსი",
        "ბორტ-რადისტი",
        "ბოტანიკოსი",
        "ბრეიდერი",
        "ბროკერი",
        "პურის მცხობელი",
        "ბულდოზერისტი",
        "ბუღალტერი",
        "გასტროენტეროლოგი",
        "გემის კაპიტანი",
        "გაუპვახტის უფროსი",
        "გამომძიებელი",
        "გალვანიკური ინჟინერი",
        "გამომცემელი",
        "გამომგონებელი",
        "გენეტიკოსი",
        "გეოგრაფი",
        "გეოდეზისტი",
        "გამყიდველი",
        "გეოლოგი",
        "გინეკოლოგი",
        "გიტარისტი",
        "გლაციოლოგი",
        "გრანატომტყორცნელი",
        "გრეიდერის ოპერატორი",
        "გრენადერი",
        "გრიმიორი",
        "გამომცემელი რედაქტორი",
        "დიზაინის ინჟინერი",
        "დეკანი",
        "დეკორატორი",
        "დურგალი",
        "დერმატოლოგი",
        "დეტექტივი",
        "დეფექტოლოგი",
        "დიჯეი",
        "დიეტოლოგი",
        "დიზაინერი",
        "დიზაინერი-კონსტრუქტორი",
        "დიქტორი",
        "დილერი",
        "დიპლომატი",
        "დიპლომატიური მუშაკი",
        "დირიჟორი",
        "დრეპერი",
        "დამლაგებელი",
        "ეკოლოგი",
        "ეკონომისტი",
        "ექსპედიტორი",
        "ექიმი ულტრაბგერითი დიაგნოსტიკის",
        "ეგვიპტოლოგი",
        "ექსპერტ-კრიმინალისტი",
        "ელექტრიკოსი",
        "ელექტროინჟინერი",
        "ენერგეტიკის ინჟინერი",
        "ენდოკრინოლოგი",
        "ენდოსკოპისტი",
        "ენტომოლოგი",
        "ეპიდემიოლოგი",
        "ეფერენტოლოგი",
        "ვებ-ინტეგრატორი",
        "ვებ-მასტერი",
        "ვებ-პროგრამისტი",
        "ვეტერინარი",
        "ვიზაჟისტი",
        "ვაზის მეღვინე",
        "ვირუსოლოგი",
        "ვოკალისტი",
        "ზეინკალი",
        "ზოოლოგი",
        "თეოლოგი",
        "თერაპევტი",
        "თერიოლოგი",
        "თიფლოპედაგოგი",
        "თორაკალური ქირურგი",
        "თარჯიმანი",
        "ილუსტრატორი",
        "იმიჯმეიკერი",
        "იმუნოლოგი",
        "იუველირი",
        "იურიდიული მრჩეველი",
        "იურისტი",
        "ინჟინერი",
        "ინკასატორი",
        "ინტენდანტი",
        "ინფექციონისტი",
        "ისტორიკოსი",
        "იქთიოლოგი",
        "კომპიუტერული ინჟინერი",
        "კინოსა და ტელევიზიის ოპერატორი",
        "კარდიოლოგი",
        "კარდიოქირურგი",
        "კასკადიორი",
        "კომპიუტერის ოპერატორი",
        "კასირი",
        "კინოლოგი",
        "კრიპტოგრაფი",
        "კინომექანიკოსი",
        "კინოოპერატორი",
        "კინორეჟისორი",
        "კლიენტთა ურთიერთობის მენეჯერი",
        "კოდერი",
        "კომენდანტი",
        "კომერციული დირექტორი",
        "კომპოზიტორი",
        "კონდიტერი",
        "კონდუქტორი",
        "კონტენტის მენეჯერი",
        "კოპირაიტერი",
        "კორექტორ-მთავარი",
        "კორექტორი",
        "კოსმეტოლოგი",
        "კოსმონავტი",
        "კრიპტოზოოლოგი",
        "კრიტიკოსი",
        "კულტუროლოგი",
        "ლაბორანტი",
        "ლიმფოლოგი",
        "ლინგვისტი",
        "ლოგოპედი",
        "ლოცმანი",
        "ლაბორანტი-ინჟინერი",
        "ლოკომოტივის მძღოლი",
        "ლექტორი",
        "მალარი",
        "მამმოლოგი",
        "მხატვარი",
        "მცხობელი",
        "მანუალისტი",
        "მარკეტოლოგი",
        "მასაჟისტი",
        "მწყემსი",
        "მანიკიურის ოსტატი",
        "მათემატიკოსი",
        "მატარებლის მძღოლი",
        "მცენარეთა სპეციალისტი",
        "მეწარმე",
        "მექანიზაციის ინჟინერი",
        "მოცეკვავე",
        "მძღოლი",
        "მსახიობი",
        "მეანე",
        "მესაზღვრე",
        "მეხანძრე",
        "მეხორბლე",
        "მიმტანი",
        "მეფუტკრე",
        "მწერალი",
        "მენეჯერი",
        "მეტალურგი",
        "მეტეოროლოგი",
        "მჭედელი",
        "მებაჟე",
        "მექანიკოსი",
        "მეცხოველეობის სპეციალისტი",
        "მეტყევე-ინჟინერი",
        "მედესანტე",
        "მექანიკის ინჟინერი",
        "მცენარეთა დაცვის აგრონომი",
        "მკერავი",
        "მტვირთავი",
        "მიკრობიოლოგი",
        "მინისტრი",
        "მოდელი",
        "მოდელიერი",
        "მებაღე",
        "მასწავლებელი",
        "მაღაროელი",
        "მეცნიერი",
        "მეწაღე",
        "მოქანდაკე",
        "მთამსვლელი",
        "მყვინთავი",
        "მეურნე",
        "მონტაჟნიკი",
        "მსაჯი",
        "მონაცემთა ბაზის ადმინისტრატორი",
        "მუსიკოსი",
        "მშენებელი",
        "ნარკოლოგი",
        "ნევროლოგი",
        "ნევროპათოლოგი",
        "ნეიროქირურგი",
        "ნეონატოლოგი",
        "ნეფროლოგი",
        "ნოტარიუსი",
        "ოკეანოლოგი",
        "ონკოლოგი",
        "ორნითოლოგი",
        "ორთოდონტი",
        "ორთოპედი",
        "ოტორინოლარინგოლოგი",
        "ოფთალმოლოგი",
        "ოჯახის ექიმი",
        "პილოტი",
        "პალეონტოლოგი",
        "პედიკიურის ოსტატი",
        "პარფიუმერი",
        "პასტორალისტი",
        "პათოლოგანატომი",
        "პედაგოგი",
        "პედიატრი",
        "პლასტიკური ქირურგი",
        "პოლიტოლოგი",
        "პოლიციელი",
        "პოეტი",
        "პროგრამისტი",
        "პროდიუსერი",
        "პირადი მცველი",
        "პროქტოლოგი",
        "პროფესიული პათოლოგი",
        "პულმონოლოგი",
        "ჟურნალისტი",
        "რეკლამის დიზაინერი",
        "რადიოლოგი",
        "რადიომექანიკოსი",
        "რადიოტელეფონისტი",
        "რადისტი",
        "რადიოლოკაციური სისტემის ოპერატორი",
        "რეანიმატოლოგი",
        "რევმატოლოგი",
        "რედაქტორი",
        "რეჟისორი",
        "რელიგიური მეცნიერი",
        "რენტგენოლოგი",
        "რესტავრატორი",
        "რეფლექსოთერაპევტი",
        "რიხტოვშიკი",
        "რობოტოტექნიკოსი",
        "სანიტარი",
        "სანტექნიკოსი",
        "საპიორი",
        "სასულიერო პირი",
        "სექსოლოგი",
        "სექსოპათოლოგი",
        "სერპენტოლოგი",
        "სისტემური ადმინისტრატორი",
        "სნაიპერი",
        "სომელიე",
        "სასაქონლო ექსპერტი",
        "სპილენძის მჭედელი",
        "სომნოლოგი",
        "სოციოლოგი",
        "სპორტული ექიმი",
        "სახელმწიფო აღმასრულებელი",
        "სამხედრო ქირურგი",
        "სამხედრო მოსამსახურე",
        "სამხედრო გამომძიებელი",
        "სამხედრო კონსულტანტი",
        "სამხედრო მთარგმნელი",
        "სამხედრო პოლიციელი",
        "საგადასახადო ინსპექტორი",
        "საწყობის უფროსი",
        "სამხედრო პროკურორი",
        "სამხედრო მოსამართლე",
        "სამედიცინო და სოციალური ექსპერტიზის ექიმი",
        "სამხედრო ინჟინერი",
        "საკრედიტო კონსულტანტი",
        "სამხედრო იურისტი",
        "სასწრაფო დახმარების ექიმი",
        "სტიუარდი",
        "სტომატოლოგი",
        "სამართალმცოდნე",
        "სადაზღვევო აგენტი",
        "სასამართლო აღსრულების სპეციალისტი",
        "სტილისტი",
        "სამხედრო ორკესტრის უფროსი",
        "სურდოლოგი",
        "სურდოპედაგოგი",
        "სცენარისტი",
        "სისტემატიკის ინჟინერი",
        "სამშენებლო ინჟინერი",
        "ტაქსის მძღოლი",
        "ტალმანი",
        "ტანატოლოგი",
        "ტანკისტი",
        "ტატუს ოსტატი",
        "ტელეგრაფისტი",
        "ტელეჟურნალისტი",
        "ტელევიზიის ოსტატი",
        "ტესტირების სპეციალისტი",
        "ტექნიკოსი",
        "ტექნოლოგი",
        "ტიპოგრაფი",
        "ტექნოლოგიური ინჟინერი",
        "ტოქსიკოლოგი",
        "ტოპოგრაფი",
        "ტრავმატოლოგი",
        "ტრაქტორისტი",
        "ტრანსფუზიოლოგი",
        "ტრედერი",
        "ტრენდ-ვოჩერი",
        "უროლოგი",
        "უმუშევარი",
        "უსაფრთხოების ტექნიკის ინჟინერი",
        "უჯრედული ტექნოლოგიების სპეციალისტი",
        "ფიზიკის ინჟინერი",
        "ფოსტალიონი",
        "ფსიქიატრი",
        "ფარმაცევტი",
        "ფიზიკოსი",
        "ფიზიოთერაპევტი",
        "ფილოლოგი",
        "ფილოსოფოსი",
        "ფინანსისტი",
        "ფლებოლოგი",
        "ფლორისტი",
        "ფოტოგრაფი",
        "ფოტომოდელი",
        "ფურაჟირი",
        "ფუტუროლოგი",
        "ფოლადის მწარმოებელი",
        "ფსიქონევროპათოლოგი",
        "ფუნქციური დიაგნოსტიკის ექიმი",
        "ფსიქოთერაპევტი",
        "ფსიქოლოგი",
        "ქვისმთლელი",
        "ქიმიის ინჟინერი",
        "ქიმიკოსი",
        "ქიმიკოს-ტექნოლოგი",
        "ქირურგი",
        "ქორეოგრაფი",
        "ქოლცენტრის ოპერატორი",
        "ქორწილების ორგანიზატორი",
        "ქვაფენილი",
        "ქუჩის დამლაგებელი",
        "ყველის მწარმოებელი",
        "შტაბის უფროსი",
        "შემკრებავი",
        "შემდუღებელი",
        "ცხვრის მჭრელი სპეციალისტი",
        "ცირკის არტისტი",
        "ცხენმშენებელი",
        "ციხის ზედამხედველი",
        "ძიძა",
        "ხარატი",
        "ხმის ინჟინერი",
        "ხელოვნებათმცოდნე",
        "ჯალათი",
        "ჰემატოლოგი",
        "ჰეპატოლოგი",
        "ჰიდროლოგი",
        "ჰომეოპათი",
        "ჰირუდოთერაპევტი",
        "ჰიდრავლიკის ინჟინერი",
    ]



from ..ar_AA import Provider as ArabicDateTimeProvider


class Provider(ArabicDateTimeProvider):
    # source: https://ar.wikipedia.org/wiki/أسماء_الشهور
    MONTH_NAMES = {
        "01": "جانفي",
        "02": "فيفري",
        "03": "مارس",
        "04": "أفريل",
        "05": "ماي",
        "06": "جوان",
        "07": "جويلية",
        "08": "أوت",
        "09": "سبتمبر",
        "10": "أكتوبر",
        "11": "نوفمبر",
        "12": "ديسمبر",
    }



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):
    price_formats = ["#.##", "%#.##", "%##.##", "%,###.##"]

    def pricetag(self) -> str:
        return "\N{NAIRA SIGN}" + "\N{NO-BREAK SPACE}" + self.numerify(self.random_element(self.price_formats))



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    safe_email_tlds = ("com", "net", "ir", "org")
    free_email_domains = (
        "chmail.ir",
        "mailfa.com",
        "gmail.com",
        "hotmail.com",
        "yahoo.com",
    )
    tlds = ("com", "com", "com", "net", "org", "ir", "ir", "ir")



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):
    """Currency provider for mk_MK locale (Macedonian).

    Sources:
    - https://mk.wikipedia.org/wiki/Валута
    - https://en.wikipedia.org/wiki/List_of_circulating_currencies
    """

    # Currency names in Macedonian
    currencies = (
        ("AED", "Дирхам на ОАЕ"),
        ("AFN", "Авганистански авганија"),
        ("ALL", "Албански лек"),
        ("AMD", "Ерменски драм"),
        ("ANG", "Нидерландски антилски гулден"),
        ("AOA", "Анголска кванза"),
        ("ARS", "Аргентинско пезо"),
        ("AUD", "Австралиски долар"),
        ("AWG", "Арубански флорин"),
        ("AZN", "Азербејџански манат"),
        ("BAM", "Конвертибилна марка на БиХ"),
        ("BBD", "Барбадоски долар"),
        ("BDT", "Бангладешка така"),
        ("BGN", "Бугарски лев"),
        ("BHD", "Бахреински динар"),
        ("BIF", "Бурундиски франк"),
        ("BMD", "Бермудски долар"),
        ("BND", "Брунејски долар"),
        ("BOB", "Боливиско боливијано"),
        ("BRL", "Бразилски реал"),
        ("BSD", "Бахамски долар"),
        ("BTN", "Бутански нгултрум"),
        ("BWP", "Ботсванска пула"),
        ("BYN", "Белоруска рубља"),
        ("BZD", "Белиски долар"),
        ("CAD", "Канадски долар"),
        ("CDF", "Конгоански франк"),
        ("CHF", "Швајцарски франк"),
        ("CLP", "Чилеанско пезо"),
        ("CNY", "Кинески јуан"),
        ("COP", "Колумбиско пезо"),
        ("CRC", "Костариканска колона"),
        ("CUP", "Кубанско пезо"),
        ("CVE", "Кабовердско ескудо"),
        ("CZK", "Чешка круна"),
        ("DJF", "Џибутиски франк"),
        ("DKK", "Данска круна"),
        ("DOP", "Доминиканско пезо"),
        ("DZD", "Алжирски динар"),
        ("EGP", "Египетска фунта"),
        ("ERN", "Еритрејска накфа"),
        ("ETB", "Етиопски бир"),
        ("EUR", "Евро"),
        ("FJD", "Фиџиски долар"),
        ("GBP", "Британска фунта"),
        ("GEL", "Грузиски лари"),
        ("GHS", "Гански седи"),
        ("GMD", "Гамбиски даласи"),
        ("GNF", "Гвинејски франк"),
        ("GTQ", "Гватемалски кецал"),
        ("GYD", "Гвајански долар"),
        ("HKD", "Хонгконшки долар"),
        ("HNL", "Хондураска лемпира"),
        ("HTG", "Хаитски гурд"),
        ("HUF", "Унгарски форинт"),
        ("IDR", "Индонезиска рупија"),
        ("ILS", "Израелски нов шекел"),
        ("INR", "Индиска рупија"),
        ("IQD", "Ирачки динар"),
        ("IRR", "Ирански риал"),
        ("ISK", "Исландска круна"),
        ("JMD", "Јамајски долар"),
        ("JOD", "Јорданоски динар"),
        ("JPY", "Јапонски јен"),
        ("KES", "Кенијски шилинг"),
        ("KGS", "Киргистански сом"),
        ("KHR", "Камбоџански риел"),
        ("KMF", "Коморски франк"),
        ("KPW", "Северна корејска вона"),
        ("KRW", "Јужна корејска вона"),
        ("KWD", "Кувајтски динар"),
        ("KZT", "Казахстански тенге"),
        ("LAK", "Лаоски кип"),
        ("LBP", "Либанска фунта"),
        ("LKR", "Шриланканска рупија"),
        ("LRD", "Либериски долар"),
        ("LYD", "Либиски динар"),
        ("MAD", "Марокански дирхам"),
        ("MDL", "Молдавски леу"),
        ("MGA", "Малагасиски ариари"),
        ("MKD", "Македонски денар"),
        ("MMK", "Мјанмарски кјат"),
        ("MNT", "Монголски тугрик"),
        ("MUR", "Маурициска рупија"),
        ("MVR", "Малдивска руфија"),
        ("MWK", "Малависка квача"),
        ("MXN", "Мексиканско пезо"),
        ("MYR", "Малезиски рингит"),
        ("MZN", "Мозамбичка метикала"),
        ("NAD", "Намибиски долар"),
        ("NGN", "Нигериска најра"),
        ("NIO", "Никарагванска кордоба"),
        ("NOK", "Норвешка круна"),
        ("NPR", "Непалска рупија"),
        ("NZD", "Новозеландски долар"),
        ("OMR", "Оманскт риал"),
        ("PAB", "Панамска балбоа"),
        ("PEN", "Перуански сол"),
        ("PHP", "Филипинско пезо"),
        ("PKR", "Пакистанска рупија"),
        ("PLN", "Полски злоти"),
        ("PYG", "Парагвајски гварани"),
        ("QAR", "Катарски риал"),
        ("RON", "Романски леу"),
        ("RSD", "Српски динар"),
        ("RUB", "Руска рубља"),
        ("RWF", "Руандски франк"),
        ("SAR", "Саудиски ријал"),
        ("SBD", "Доларот на Соломоновите Острови"),
        ("SCR", "Сејшелска рупија"),
        ("SDG", "Судански фунт"),
        ("SEK", "Шведска круна"),
        ("SGD", "Сингапурски долар"),
        ("SLL", "Сиера Леонски леоне"),
        ("SOS", "Сомалиски шилинг"),
        ("SRD", "Суринамски долар"),
        ("SYP", "Сириска фунта"),
        ("SZL", "Свазиленд лилангени"),
        ("THB", "Тајландски бат"),
        ("TJS", "Таџикистански сомони"),
        ("TMT", "Туркменистански манат"),
        ("TND", "Тунизиски динар"),
        ("TRY", "Турска лира"),
        ("TTD", "Тринидадски долар"),
        ("TWD", "Нов тајвански долар"),
        ("TZS", "Танзаниски шилинг"),
        ("UAH", "Украинска хривна"),
        ("UGX", "Угандиски шилинг"),
        ("USD", "Американски долар"),
        ("UYU", "Уругвајско пезо"),
        ("UZS", "Узбекистански сум"),
        ("VES", "Венецуелански боливар"),
        ("VND", "Виетнамски донг"),
        ("VUV", "Вануатски вату"),
        ("WST", "Самоанска тала"),
        ("XAF", "Централноафрикански франк CFA"),
        ("XCD", "Источнокарипски долар"),
        ("XOF", "Западноафрикански франк CFA"),
        ("XPF", "Франк CFP"),
        ("YER", "Јеменски риал"),
        ("ZAR", "Јужноафрикански ренд"),
        ("ZMW", "Замбиска квача"),
        ("ZWL", "Зимбабвески долар"),
    )

    price_formats = ("# ден.", "## ден.", "### ден.", "#.### ден.", "##.### ден.")

    def pricetag(self) -> str:
        return self.numerify(self.random_element(self.price_formats))



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    user_name_formats = (
        "{{last_name_female}}.{{first_name_female}}",
        "{{last_name_male}}.{{first_name_male}}",
        "{{first_name_female}}.{{last_name_female}}",
        "{{first_name_male}}.{{last_name_male}}",
        "{{first_name}}##",
        "?{{last_name}}",
        "?{{last_name}}",
        "?{{last_name}}",
    )

    email_formats = ("{{user_name}}@{{free_email_domain}}",)

    free_email_domains = (
        "email.ro",
        "gmail.com",
        "kappa.ro",
        "acasa.ro",
        "zzup.ro",
        "141.ro",
        "post.ro",
    )

    tlds = ("ro", "com", "ro")



from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    """Implement phone_number provider for ``vi_VN`` locale.
    # Source : https://vi.wikipedia.org/wiki/M%C3%A3_%C4%91i%E1%BB%87n_tho%E1%BA%A1i_Vi%E1%BB%87t_Nam
    """

    formats = (
        "+84 ## #######",
        "(0#) #### ####",
        "0# #### ####",
        "0# #######",
        "+84-##-######",
        "+84-##-### ####",
        "(0#)###-####",
    )



from typing import Dict

from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``es_ES`` locale.

    Sources:

    - https://corpus.rae.es/frec/1000_formas.TXT
    """

    word_list = (
        "de",
        "la",
        "que",
        "el",
        "en",
        "y",
        "a",
        "los",
        "se",
        "del",
        "las",
        "un",
        "por",
        "con",
        "no",
        "una",
        "su",
        "para",
        "es",
        "al",
        "lo",
        "como",
        "más",
        "o",
        "pero",
        "sus",
        "le",
        "ha",
        "me",
        "si",
        "sin",
        "sobre",
        "este",
        "ya",
        "entre",
        "cuando",
        "todo",
        "esta",
        "ser",
        "son",
        "dos",
        "también",
        "fue",
        "había",
        "era",
        "muy",
        "años",
        "hasta",
        "desde",
        "está",
        "mi",
        "porque",
        "qué",
        "sólo",
        "han",
        "yo",
        "hay",
        "vez",
        "puede",
        "todos",
        "así",
        "nos",
        "ni",
        "parte",
        "tiene",
        "él",
        "uno",
        "donde",
        "bien",
        "tiempo",
        "mismo",
        "ese",
        "ahora",
        "cada",
        "e",
        "vida",
        "otro",
        "después",
        "te",
        "otros",
        "aunque",
        "esa",
        "eso",
        "hace",
        "otra",
        "gobierno",
        "tan",
        "durante",
        "siempre",
        "día",
        "tanto",
        "ella",
        "tres",
        "sí",
        "dijo",
        "sido",
        "gran",
        "país",
        "según",
        "menos",
        "mundo",
        "año",
        "antes",
        "estado",
        "contra",
        "sino",
        "forma",
        "caso",
        "nada",
        "hacer",
        "general",
        "estaba",
        "poco",
        "estos",
        "presidente",
        "mayor",
        "ante",
        "unos",
        "les",
        "algo",
        "hacia",
        "casa",
        "ellos",
        "ayer",
        "hecho",
        "primera",
        "mucho",
        "mientras",
        "además",
        "quien",
        "momento",
        "millones",
        "esto",
        "españa",
        "hombre",
        "están",
        "pues",
        "hoy",
        "lugar",
        "madrid",
        "nacional",
        "trabajo",
        "otras",
        "mejor",
        "nuevo",
        "decir",
        "algunos",
        "entonces",
        "todas",
        "días",
        "debe",
        "política",
        "cómo",
        "casi",
        "toda",
        "tal",
        "luego",
        "pasado",
        "primer",
        "medio",
        "va",
        "estas",
        "sea",
        "tenía",
        "nunca",
        "poder",
        "aquí",
        "ver",
        "veces",
        "embargo",
        "partido",
        "personas",
        "grupo",
        "cuenta",
        "pueden",
        "tienen",
        "misma",
        "nueva",
        "cual",
        "fueron",
        "mujer",
        "frente",
        "josé",
        "tras",
        "cosas",
        "fin",
        "ciudad",
        "he",
        "social",
        "manera",
        "tener",
        "sistema",
        "será",
        "historia",
        "muchos",
        "juan",
        "tipo",
        "cuatro",
        "dentro",
        "nuestro",
        "punto",
        "dice",
        "ello",
        "cualquier",
        "noche",
        "aún",
        "agua",
        "parece",
        "haber",
        "situación",
        "fuera",
        "bajo",
        "grandes",
        "nuestra",
        "ejemplo",
        "acuerdo",
        "habían",
        "usted",
        "estados",
        "hizo",
        "nadie",
        "países",
        "horas",
        "posible",
        "tarde",
        "ley",
        "importante",
        "guerra",
        "desarrollo",
        "proceso",
        "realidad",
        "sentido",
        "lado",
        "mí",
        "tu",
        "cambio",
        "allí",
        "mano",
        "eran",
        "estar",
        "san",
        "número",
        "sociedad",
        "unas",
        "centro",
        "padre",
        "gente",
        "final",
        "relación",
        "cuerpo",
        "obra",
        "incluso",
        "través",
        "último",
        "madre",
        "mis",
        "modo",
        "problemas",
        "cinco",
        "carlos",
        "hombres",
        "información",
        "ojos",
        "muerte",
        "nombre",
        "algunas",
        "público",
        "mujeres",
        "siglo",
        "todavía",
        "meses",
        "mañana",
        "esos",
        "nosotros",
        "hora",
        "muchas",
        "pueblo",
        "alguna",
        "dar",
        "problema",
        "don",
        "da",
        "tú",
        "derecho",
        "verdad",
        "maría",
        "unidos",
        "podría",
        "sería",
        "junto",
        "cabeza",
        "aquel",
        "luis",
        "cuanto",
        "tierra",
        "equipo",
        "segundo",
        "director",
        "dicho",
        "cierto",
        "casos",
        "manos",
        "nivel",
        "podía",
        "familia",
        "largo",
        "partir",
        "falta",
        "llegar",
        "propio",
        "ministro",
        "cosa",
        "primero",
        "seguridad",
        "hemos",
        "mal",
        "trata",
        "algún",
        "tuvo",
        "respecto",
        "semana",
        "varios",
        "real",
        "sé",
        "voz",
        "paso",
        "señor",
        "mil",
        "quienes",
        "proyecto",
        "mercado",
        "mayoría",
        "luz",
        "claro",
        "iba",
        "éste",
        "pesetas",
        "orden",
        "español",
        "buena",
        "quiere",
        "aquella",
        "programa",
        "palabras",
        "internacional",
        "van",
        "esas",
        "segunda",
        "empresa",
        "puesto",
        "ahí",
        "propia",
        "m",
        "libro",
        "igual",
        "político",
        "persona",
        "últimos",
        "ellas",
        "total",
        "creo",
        "tengo",
        "dios",
        "c",
        "española",
        "condiciones",
        "méxico",
        "fuerza",
        "solo",
        "único",
        "acción",
        "amor",
        "policía",
        "puerta",
        "pesar",
        "zona",
        "sabe",
        "calle",
        "interior",
        "tampoco",
        "música",
        "ningún",
        "vista",
        "campo",
        "buen",
        "hubiera",
        "saber",
        "obras",
        "razón",
        "ex",
        "niños",
        "presencia",
        "tema",
        "dinero",
        "comisión",
        "antonio",
        "servicio",
        "hijo",
        "última",
        "ciento",
        "estoy",
        "hablar",
        "dio",
        "minutos",
        "producción",
        "camino",
        "seis",
        "quién",
        "fondo",
        "dirección",
        "papel",
        "demás",
        "barcelona",
        "idea",
        "especial",
        "diferentes",
        "dado",
        "base",
        "capital",
        "ambos",
        "europa",
        "libertad",
        "relaciones",
        "espacio",
        "medios",
        "ir",
        "actual",
        "población",
        "empresas",
        "estudio",
        "salud",
        "servicios",
        "haya",
        "principio",
        "siendo",
        "cultura",
        "anterior",
        "alto",
        "media",
        "mediante",
        "primeros",
        "arte",
        "paz",
        "sector",
        "imagen",
        "medida",
        "deben",
        "datos",
        "consejo",
        "personal",
        "interés",
        "julio",
        "grupos",
        "miembros",
        "ninguna",
        "existe",
        "cara",
        "edad",
        "etc",
        "movimiento",
        "visto",
        "llegó",
        "puntos",
        "actividad",
        "bueno",
        "uso",
        "niño",
        "difícil",
        "joven",
        "futuro",
        "aquellos",
        "mes",
        "pronto",
        "soy",
        "hacía",
        "nuevos",
        "nuestros",
        "estaban",
        "posibilidad",
        "sigue",
        "cerca",
        "resultados",
        "educación",
        "atención",
        "gonzález",
        "capacidad",
        "efecto",
        "necesario",
        "valor",
        "aire",
        "investigación",
        "siguiente",
        "figura",
        "central",
        "comunidad",
        "necesidad",
        "serie",
        "organización",
        "nuevas",
        "calidad",
        "economía",
        "carácter",
        "jefe",
        "estamos",
        "prensa",
        "control",
        "sociales",
        "universidad",
        "militar",
        "cabo",
        "diez",
        "fuerzas",
        "congreso",
        "ésta",
        "hijos",
        "justicia",
        "mundial",
        "dólares",
        "juego",
        "económica",
        "políticos",
        "duda",
        "recursos",
        "pública",
        "crisis",
        "próximo",
        "tenemos",
        "decisión",
        "varias",
        "popular",
        "tenido",
        "apenas",
        "época",
        "banco",
        "presente",
        "menor",
        "quiero",
        "pasar",
        "resultado",
        "televisión",
        "encuentra",
        "gracias",
        "ministerio",
        "conjunto",
        "defensa",
        "alguien",
        "queda",
        "hacen",
        "pasa",
        "resto",
        "causa",
        "seguir",
        "allá",
        "palabra",
        "voy",
        "cuya",
        "vamos",
        "mar",
        "estudios",
        "derechos",
        "importancia",
        "cuales",
        "contrario",
        "manuel",
        "garcía",
        "fuerte",
        "sol",
        "jóvenes",
        "apoyo",
        "habría",
        "civil",
        "miguel",
        "pedro",
        "partidos",
        "libre",
        "fuentes",
        "administración",
        "común",
        "dejar",
        "cine",
        "salir",
        "comunicación",
        "b",
        "experiencia",
        "demasiado",
        "plan",
        "respuesta",
        "energía",
        "izquierda",
        "función",
        "principal",
        "superior",
        "naturaleza",
        "podemos",
        "unión",
        "especialmente",
        "rey",
        "domingo",
        "favor",
        "cantidad",
        "elecciones",
        "clase",
        "productos",
        "españoles",
        "conocer",
        "teatro",
        "importantes",
        "evitar",
        "color",
        "actividades",
        "mesa",
        "p",
        "decía",
        "cuyo",
        "debido",
        "alta",
        "francisco",
        "secretario",
        "objeto",
        "quizá",
        "posición",
        "parecía",
        "natural",
        "elementos",
        "hubo",
        "objetivo",
        "formas",
        "única",
        "pueda",
        "origen",
        "blanco",
        "mismos",
        "lleva",
        "económico",
        "opinión",
        "ayuda",
        "oficial",
        "silencio",
        "buenos",
        "pensar",
        "república",
        "dónde",
        "sangre",
        "encuentro",
        "siquiera",
        "autor",
        "reunión",
        "haciendo",
        "suelo",
        "muestra",
        "viejo",
        "encima",
        "resulta",
        "tomar",
        "bastante",
        "siete",
        "lucha",
        "pudo",
        "amigos",
        "línea",
        "sur",
        "pocos",
        "medidas",
        "norte",
        "partes",
        "iglesia",
        "tratamiento",
        "existencia",
        "cargo",
        "grande",
        "américa",
        "boca",
        "plaza",
        "pie",
        "trabajadores",
        "poner",
        "existen",
        "viene",
        "permite",
        "análisis",
        "argentina",
        "acto",
        "hechos",
        "tiempos",
        "políticas",
        "radio",
        "puedo",
        "crecimiento",
        "francia",
        "compañía",
        "amigo",
        "autoridades",
        "realizar",
        "acciones",
        "padres",
        "diario",
        "ve",
        "derecha",
        "ambiente",
        "i",
        "habrá",
        "precisamente",
        "enfermedad",
        "especie",
        "ejército",
        "santa",
        "cambios",
        "río",
        "sabía",
        "seguro",
        "espera",
        "momentos",
        "viaje",
        "quería",
        "ocho",
        "vivir",
        "región",
        "formación",
        "escuela",
        "cuarto",
        "valores",
        "quedó",
        "participación",
        "éxito",
        "baja",
        "artículo",
        "principales",
        "fernando",
        "metros",
        "marcha",
        "régimen",
        "consecuencia",
        "conocimiento",
        "corazón",
        "campaña",
        "estructura",
        "efectos",
        "finalmente",
        "modelo",
        "carta",
        "construcción",
        "médico",
        "miedo",
        "mayores",
        "entrada",
        "humanos",
        "sean",
        "actitud",
        "deja",
        "dejó",
        "d",
        "llevar",
        "negro",
        "texto",
        "mitad",
        "estuvo",
        "alrededor",
        "acerca",
        "peso",
        "humano",
        "pequeño",
        "fecha",
        "serán",
        "doctor",
        "ideas",
        "vino",
        "materia",
        "llega",
        "carrera",
        "cierta",
        "sola",
        "psoe",
        "lejos",
        "juez",
        "características",
        "riesgo",
        "fácil",
        "diferencia",
        "cultural",
        "libros",
        "práctica",
        "mayo",
        "nuestras",
        "programas",
        "memoria",
        "llegado",
        "plazo",
        "expresión",
        "diciembre",
        "mantener",
        "enero",
        "volver",
        "cuadro",
        "producto",
        "produce",
        "europea",
        "conciencia",
        "tenían",
        "atrás",
        "felipe",
        "creación",
        "chile",
        "precio",
        "película",
        "puerto",
        "fuego",
        "cuestión",
        "pasó",
        "costa",
        "supuesto",
        "local",
        "habla",
        "aspectos",
        "cuba",
        "sala",
        "cámara",
        "vuelta",
        "vía",
        "mirada",
        "mejores",
        "informe",
        "unidad",
        "distintos",
        "suerte",
        "tales",
        "mira",
        "llamado",
        "técnica",
        "título",
        "s",
        "principios",
        "octubre",
        "volvió",
        "período",
        "g",
        "encontrar",
        "democracia",
        "aumento",
        "fútbol",
        "prueba",
        "consumo",
        "pese",
        "ocasiones",
        "exterior",
        "solución",
        "u",
        "hija",
        "sueño",
        "parís",
        "capaz",
        "ocasión",
        "industria",
        "adelante",
        "salida",
        "ciencia",
        "asunto",
        "asociación",
        "puso",
        "intereses",
        "oro",
        "podrá",
        "pregunta",
        "oposición",
        "entrar",
        "señora",
        "señaló",
        "santiago",
        "dolor",
        "zonas",
        "comercio",
        "operación",
        "tribunal",
        "instituciones",
        "temas",
        "militares",
        "junio",
        "marco",
        "sectores",
        "hacerlo",
        "aspecto",
        "razones",
        "contenido",
        "juicio",
        "electoral",
        "considera",
        "tendrá",
        "mucha",
        "voluntad",
        "dicen",
        "recuerdo",
        "socialista",
        "área",
        "aparece",
        "vio",
        "cama",
        "aun",
        "presenta",
        "pp",
        "revolución",
        "busca",
        "abril",
        "rodríguez",
        "fiscal",
        "lópez",
        "victoria",
        "violencia",
        "primeras",
        "pequeña",
        "armas",
        "debía",
        "ii",
        "esfuerzo",
        "humana",
        "posibilidades",
        "centros",
        "profesional",
        "asimismo",
        "grado",
        "has",
        "toma",
        "distintas",
        "material",
        "carne",
        "llama",
        "particular",
        "jorge",
        "trabajar",
        "propuesta",
        "muerto",
        "precios",
        "reforma",
        "hermano",
        "corte",
        "comenzó",
        "etapa",
        "obstante",
        "pone",
        "diversos",
        "visita",
        "concepto",
        "pacientes",
        "semanas",
        "tipos",
        "solamente",
        "deseo",
        "sistemas",
        "encuentran",
        "siguientes",
        "martín",
        "suficiente",
        "marzo",
        "propios",
        "jamás",
        "dan",
        "club",
        "instituto",
        "constitución",
        "curso",
        "lenguaje",
        "estilo",
        "rosa",
        "imposible",
        "pablo",
        "buscar",
        "peor",
        "piel",
        "arriba",
        "generales",
        "septiembre",
        "blanca",
        "r",
        "aquellas",
        "teoría",
        "animales",
        "hicieron",
        "larga",
        "perdido",
        "imágenes",
        "paciente",
        "conseguir",
        "máximo",
        "noviembre",
        "j",
        "líder",
        "hospital",
        "diversas",
        "rafael",
        "vuelve",
        "destino",
        "torno",
        "proyectos",
        "flores",
        "niveles",
        "afirmó",
        "explicó",
        "n",
        "somos",
        "términos",
        "premio",
    )

    parts_of_speech: Dict[str, tuple] = {}



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    """Address provider for mk_MK locale (Macedonian).

    Sources:
    - https://en.wikipedia.org/wiki/List_of_cities_in_North_Macedonia
    - https://en.wikipedia.org/wiki/List_of_sovereign_states
    """

    city_formats = ("{{city_name}}",)

    street_name_formats = ("{{street_name}}",)
    street_address_formats = ("{{street_name}} {{building_number}}",)
    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)

    building_number_formats = (
        "###",
        "##",
        "#",
        "#а",
        "#б",
        "#в",
        "#а/#",
        "#б/#",
    )

    postcode_formats = ("#####",)

    street_suffixes_long = (
        "",
        "улица",
        "булевар",
        "пат",
        "авенија",
    )

    street_suffixes_short = (
        "",
        "ул.",
        "бул.",
        "п.",
        "а.",
    )

    cities = (
        "Берово",  # Berovo
        "Битола",  # Bitola
        "Богданци",  # Bogdanci
        "Валандово",  # Valandovo
        "Велес",  # Veles
        "Виница",  # Vinica
        "Гевгелија",  # Gevgelija
        "Гостивар",  # Gostivar
        "Дебар",  # Debar
        "Делчево",  # Delčevo
        "Демир Капија",  # Demir Kapija
        "Демир Хисар",  # Demir Hisar
        "Кавадарци",  # Kavadarci
        "Кичево",  # Kičevo
        "Кочани",  # Kočani
        "Кратово",  # Kratovo
        "Крива Паланка",  # Kriva Palanka
        "Крушево",  # Kruševo
        "Куманово",  # Kumanovo
        "Македонска Каменица",  # Makedonska Kamenica
        "Македонски Брод",  # Makedonski Brod
        "Неготино",  # Negotino
        "Охрид",  # Ohrid
        "Пехчево",  # Pehčevo
        "Прилеп",  # Prilep
        "Пробиштип",  # Probištip
        "Радовиш",  # Radoviš
        "Ресен",  # Resen
        "Свети Николе",  # Sveti Nikole
        "Скопје",  # Skopje
        "Струга",  # Struga
        "Струмица",  # Strumica
        "Тетово",  # Tetovo
        "Штип",  # Štip
    )

    # Source: OpenStreetMap Overpass API — named roads in Скопје
    streets = (
        "Македонија",
        "Партизански одреди",
        "Илинденска",
        "Димитар Влахов",
        "Кеј Димитар Влахов",
        "Кеј Македонија",
        "Орце Николов",
        "Никола Карев",
        "Климент Охридски",
        "Крсте Мисирков",
        "Гоце Делчев",
        "Даме Груев",
        "Јане Сандански",
        "Христијан Тодоровски Карпош",
        "Питу Гули",
        "Тодор Александров",
        "Свети Климент Охридски",
        "Благој Давков",
        "Наум Охридски",
        "Булевар Александар Македонски",
        "Булевар Блаже Конески",
        "Булевар Гоце Делчев",
        "Булевар Никола Карев",
        "Булевар Партизански одреди",
        "Булевар Свети Климент Охридски",
        "Булевар Цветан Димов",
        "Самоилова",
        "Глигор Прличев",
        "Ибрахим Темо",
        "Васил Главинов",
        "Никола Вапцаров",
        "Маршал Тито",
        "Едвард Кардељ",
        "Георги Димитров",
        "Михаило Апостолски",
        "Михаило Пупин",
        "Перо Наков",
        "Петар Поп Арсов",
        "Јован Хаџи Константинов Џинот",
        "Кирил Пејчиновиќ",
        "Лазар Личеноски",
        "Никола Русински",
        "Аристотел",
        "Луј Пастер",
        "Николај Коперник",
        "Стоби",
        "Скупи",
        "Дојранска",
        "Воденска",
        "Софиска",
        "Варшавска",
        "Виенска",
        "Женевска",
        "Лондонска",
        "Букурешка",
        "Будимпештанска",
        "Хелсинки",
        "Бриселска",
        "Ленинградска",
        "Волгоградска",
        "Крушевска",
        "Радовишка",
        "Трета македонска бригада",
        "Пролетерска",
        "Востаничка",
        "Одбранбена",
        "Зелезничка",
        "Рибарска",
        "Јасминска",
        "Природна",
        "Шарпланинска",
        "Славеј Планина",
    )

    states = (
        "Скопски",
        "Пелагониски",
        "Полошки",
        "Вардарски",
        "Источен",
        "Западен",
        "Југозападен",
        "Југоисточен",
        "Североисточен",
        "Централен",
    )

    countries = (
        "Авганистан",
        "Австралија",
        "Австрија",
        "Азербејџан",
        "Албанија",
        "Алжир",
        "Андора",
        "Ангола",
        "Антигва и Барбуда",
        "Аргентина",
        "Ерменија",
        "Бахами",
        "Бахреин",
        "Бангладеш",
        "Барбадос",
        "Белорусија",
        "Белгија",
        "Белизе",
        "Бенин",
        "Бутан",
        "Боливија",
        "Босна и Херцеговина",
        "Боцвана",
        "Бразил",
        "Брунеј",
        "Бугарија",
        "Буркина Фасо",
        "Бурунди",
        "Кабо Верде",
        "Камбоџа",
        "Камерун",
        "Канада",
        "Централноафриканска Република",
        "Чад",
        "Чиле",
        "Кина",
        "Колумбија",
        "Коморски Острови",
        "Демократска Република Конго",
        "Република Конго",
        "Коста Рика",
        "Кот д'Ивоар",
        "Хрватска",
        "Куба",
        "Кипар",
        "Чешка",
        "Данска",
        "Џибути",
        "Доминика",
        "Доминиканска Република",
        "Источен Тимор",
        "Еквадор",
        "Египет",
        "Ел Салвадор",
        "Екваторска Гвинеја",
        "Еритреја",
        "Естонија",
        "Есватини",
        "Етиопија",
        "Фиџи",
        "Финска",
        "Франција",
        "Габон",
        "Гамбија",
        "Грузија",
        "Германија",
        "Гана",
        "Грција",
        "Гренада",
        "Гватемала",
        "Гвинеја",
        "Гвинеја Бисао",
        "Гајана",
        "Хаити",
        "Хондурас",
        "Унгарија",
        "Исланд",
        "Индија",
        "Индонезија",
        "Иран",
        "Ирак",
        "Ирска",
        "Израел",
        "Италија",
        "Јамајка",
        "Јапонија",
        "Јордан",
        "Казахстан",
        "Кенија",
        "Кирибати",
        "Северна Кореја",
        "Јужна Кореја",
        "Косово",
        "Кувајт",
        "Киргистан",
        "Лаос",
        "Латвија",
        "Либан",
        "Лесото",
        "Либерија",
        "Либија",
        "Лихтенштајн",
        "Литванија",
        "Луксембург",
        "Мадагаскар",
        "Малави",
        "Малезија",
        "Малдиви",
        "Мали",
        "Малта",
        "Маршалски Острови",
        "Мавританија",
        "Маврициус",
        "Мексико",
        "Микронезија",
        "Молдавија",
        "Монако",
        "Монголија",
        "Черна Гора",
        "Мароко",
        "Мозамбик",
        "Мјанмар",
        "Намибија",
        "Науру",
        "Непал",
        "Холандија",
        "Нов Зеланд",
        "Никарагва",
        "Нигер",
        "Нигерија",
        "Северна Македонија",
        "Норвешка",
        "Оман",
        "Пакистан",
        "Палау",
        "Панама",
        "Папуа Нова Гвинеја",
        "Парагвај",
        "Перу",
        "Филипини",
        "Полска",
        "Португалија",
        "Катар",
        "Романија",
        "Русија",
        "Руанда",
        "Сент Китс и Невис",
        "Света Луција",
        "Сент Винсент и Гренадини",
        "Самоа",
        "Сан Марино",
        "Саудиска Арабија",
        "Сенегал",
        "Србија",
        "Сејшели",
        "Сиера Леоне",
        "Сингапур",
        "Словачка",
        "Словенија",
        "Соломонски Острови",
        "Сомалија",
        "Јужна Африка",
        "Јужен Судан",
        "Шпанија",
        "Шри Ланка",
        "Судан",
        "Суринам",
        "Шведска",
        "Швајцарија",
        "Сирија",
        "Тајван",
        "Таџикистан",
        "Танзанија",
        "Тајланд",
        "Того",
        "Тонга",
        "Тринидад и Тобаго",
        "Тунис",
        "Турција",
        "Туркменистан",
        "Тувалу",
        "Уганда",
        "Украина",
        "Обединети Арапски Емирати",
        "Обединето Кралство",
        "Соединети Американски Држави",
        "Уругвај",
        "Узбекистан",
        "Вануату",
        "Ватикан",
        "Венецуела",
        "Виетнам",
        "Јемен",
        "Замбија",
        "Зимбабве",
    )

    def city_name(self) -> str:
        return self.random_element(self.cities)

    def street_name(self) -> str:
        return self.random_element(self.streets)

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit



from collections import OrderedDict

from faker.typing import OrderedDictType

from ..de import Provider as BaseProvider


class Provider(BaseProvider):
    all_colors: OrderedDictType[str, str] = OrderedDict(
        (color_name.replace("ß", "ss"), color_hexcode) for color_name, color_hexcode in BaseProvider.all_colors.items()
    )



from .. import Provider as SsnProvider


class Provider(SsnProvider):
    """SSN provider for mk_MK locale (Macedonian).

    The Macedonian EMBG (Единствен матичен број на граѓанинот) is a 13-digit
    unique citizen identification number with the format:

        DDMMYYYRRGBBK

    where:
        DD    - day of birth (01-31)
        MM    - month of birth (01-12)
        YYY   - last 3 digits of year of birth
        RR    - region code (North Macedonia uses 41)
        G     - gender and century digit (0-4 female, 5-9 male; odd=1900s, even=2000s)
        BB    - sequence number within region/gender/day (00-99)
        K     - checksum digit (MOD 11)
    """

    # North Macedonia region code
    REGION_CODE = "41"

    @staticmethod
    def _checksum(digits: list) -> int:
        weights = [7, 6, 5, 4, 3, 2, 7, 6, 5, 4, 3, 2]
        total = sum(w * d for w, d in zip(weights, digits))
        remainder = 11 - (total % 11)
        if remainder in (10, 11):
            return 0
        return remainder

    def ssn(self) -> str:
        """Generate a valid Macedonian EMBG number."""
        rng = self.generator.random

        # Random birth date between 1950 and 2005
        birth_year = rng.randint(1950, 2005)
        birth_month = rng.randint(1, 12)
        # Simplified: use day range 1-28 to avoid invalid dates
        birth_day = rng.randint(1, 28)

        dd = f"{birth_day:02d}"
        mm = f"{birth_month:02d}"
        yyy = f"{birth_year % 1000:03d}"

        rr = self.REGION_CODE

        # Gender: 0-4 female (born 1900s: odd, 2000s: even), 5-9 male
        is_male = rng.choice([True, False])
        if is_male:
            g = rng.randint(5, 9)
        else:
            g = rng.randint(0, 4)

        bb = f"{rng.randint(0, 99):02d}"

        digits = [int(c) for c in dd + mm + yyy + rr + str(g) + bb]
        k = self._checksum(digits)

        return dd + mm + yyy + rr + str(g) + bb + str(k)
